@@ -21,6 +21,15 @@
 
 #include "fsmgraph.h"
 
+#ifdef LOG_CONDS
+#include <iostream>
+using std::cerr;
+using std::endl;
+#endif
+
+CondData *condData = 0;
+KeyOps *keyOps = 0;
+
 /* Insert an action into an action table. */
 void ActionTable::setAction( int ordering, Action *action )
 {
@@ -100,14 +109,6 @@ void PriorTable::setPriors( const PriorTable &other )
 		setPrior( priorIt->ordering, priorIt->desc );
 }
 
-/* If there is a state dict element, then delete it. Everything else is left
- * up to the FsmGraph destructor. */
-StateAp::~StateAp()
-{
-	if ( stateDictEl != 0 )
-		delete stateDictEl;
-}
-
 /* Set the priority of starting transitions. Isolates the start state so it has
  * no other entry points, then sets the priorities of all the transitions out
  * of the start state. If the start state is final, then the outPrior of the
@@ -120,10 +121,9 @@ void FsmAp::startFsmPrior( int ordering, PriorDesc *prior )
 	isolateStartState();
 
 	/* Walk all transitions out of the start state. */
-	OutIter outIt( startState );
-	for ( ; ! outIt.end(); outIt++ ) {
-		if ( outIt.trans->toState != 0 )
-			outIt.trans->priorTable.setPrior( ordering, prior );
+	for ( TransList::Iter trans = startState->outList; trans.lte(); trans++ ) {
+		if ( trans->toState != 0 )
+			trans->priorTable.setPrior( ordering, prior );
 	}
 }
 
@@ -134,10 +134,9 @@ void FsmAp::allTransPrior( int ordering, PriorDesc *prior )
 	/* Walk the list of all states. */
 	for ( StateList::Iter state = stateList; state.lte(); state++ ) {
 		/* Walk the out list of the state. */
-		OutIter outIt( state );
-		for ( ; ! outIt.end() ; outIt++ ) {
-			if ( outIt.trans->toState != 0 )
-				outIt.trans->priorTable.setPrior( ordering, prior );
+		for ( TransList::Iter trans = state->outList; trans.lte(); trans++ ) {
+			if ( trans->toState != 0 )
+				trans->priorTable.setPrior( ordering, prior );
 		}
 	}
 }
@@ -152,9 +151,8 @@ void FsmAp::finishFsmPrior( int ordering, PriorDesc *prior )
 	/* Walk all final states. */
 	for ( StateSet::Iter state = finStateSet; state.lte(); state++ ) {
 		/* Walk all in transitions of the final state. */
-		InIter inIt( *state );
-		for ( ; ! inIt.end(); inIt++ )
-			inIt.trans->priorTable.setPrior( ordering, prior );
+		for ( TransInList::Iter trans = (*state)->inList; trans.lte(); trans++ )
+			trans->priorTable.setPrior( ordering, prior );
 	}
 }
 
@@ -181,9 +179,9 @@ void FsmAp::startFsmAction( int ordering, Action *action )
 	isolateStartState();
 
 	/* Walk the start state's transitions, setting functions. */
-	for ( OutIter outIt( startState ); ! outIt.end(); outIt++ ) {
-		if ( outIt.trans->toState != 0 )
-			outIt.trans->actionTable.setAction( ordering, action );
+	for ( TransList::Iter trans = startState->outList; trans.lte(); trans++ ) {
+		if ( trans->toState != 0 )
+			trans->actionTable.setAction( ordering, action );
 	}
 }
 
@@ -194,9 +192,9 @@ void FsmAp::allTransAction( int ordering, Action *action )
 	/* Walk all states. */
 	for ( StateList::Iter state = stateList; state.lte(); state++ ) {
 		/* Walk the out list of the state. */
-		for ( OutIter outIt( state ); ! outIt.end(); outIt++ ) {
-			if ( outIt.trans->toState != 0 )
-				outIt.trans->actionTable.setAction( ordering, action );
+		for ( TransList::Iter trans = state->outList; trans.lte(); trans++ ) {
+			if ( trans->toState != 0 )
+				trans->actionTable.setAction( ordering, action );
 		}
 	}
 }
@@ -210,8 +208,8 @@ void FsmAp::finishFsmAction( int ordering, Action *action )
 	/* Walk all final states. */
 	for ( StateSet::Iter state = finStateSet; state.lte(); state++ ) {
 		/* Walk the final state's in list. */
-		for ( InIter inIt( *state ); ! inIt.end(); inIt++ )
-			inIt.trans->actionTable.setAction( ordering, action );
+		for ( TransInList::Iter trans = (*state)->inList; trans.lte(); trans++ )
+			trans->actionTable.setAction( ordering, action );
 	}
 }
 
@@ -230,61 +228,96 @@ void FsmAp::longMatchAction( int ordering, LongestMatchPart *lmPart )
 	/* Walk all final states. */
 	for ( StateSet::Iter state = finStateSet; state.lte(); state++ ) {
 		/* Walk the final state's in list. */
-		for ( InIter inIt( *state ); ! inIt.end(); inIt++ )
-			inIt.trans->lmActionTable.setAction( ordering, lmPart );
+		for ( TransInList::Iter trans = (*state)->inList; trans.lte(); trans++ )
+			trans->lmActionTable.setAction( ordering, lmPart );
+	}
+}
+
+void FsmAp::fillGaps( StateAp *state )
+{
+	if ( state->outList.length() == 0 ) {
+		/* Add the range on the lower and upper bound. */
+		attachNewTrans( state, 0, keyOps->lowKey, keyOps->highKey );
+	}
+	else {
+		TransList srcList;
+		srcList.transfer( state->outList );
+
+		/* Check for a gap at the beginning. */
+		TransList::Iter trans = srcList, next;
+		if ( keyOps->lt( keyOps->lowKey, trans->lowKey ) ) {
+			/* Make the high key and append. */
+			long highKey = trans->lowKey;
+			keyOps->dec( highKey );
+
+			attachNewTrans( state, 0, keyOps->lowKey, highKey );
+		}
+
+		/* Write the transition. */
+		next = trans.next();
+		state->outList.append( trans );
+
+		/* Keep the last high end. */
+		long lastHigh = trans->highKey;
+
+		/* Loop each source range. */
+		for ( trans = next; trans.lte(); trans = next ) {
+			/* Make the next key following the last range. */
+			long nextKey = lastHigh;
+			keyOps->inc( nextKey );
+
+			/* Check for a gap from last up to here. */
+			if ( keyOps->lt( nextKey, trans->lowKey ) ) {
+				/* Make the high end of the range that fills the gap. */
+				long highKey = trans->lowKey;
+				keyOps->dec( highKey );
+
+				attachNewTrans( state, 0, nextKey, highKey );
+			}
+
+			/* Reduce the transition. If it reduced to anything then add it. */
+			next = trans.next();
+			state->outList.append( trans );
+
+			/* Keep the last high end. */
+			lastHigh = trans->highKey;
+		}
+
+		/* Now check for a gap on the end to fill. */
+		if ( keyOps->lt( lastHigh, keyOps->highKey ) ) {
+			/* Get a copy of the default. */
+			keyOps->inc( lastHigh );
+
+			attachNewTrans( state, 0, lastHigh, keyOps->highKey );
+		}
 	}
 }
 
 void FsmAp::setErrorAction( StateAp *state, int ordering, Action *action )
 {
+	/* Fill any gaps in the out list with an error transition. */
+	fillGaps( state );
+
 	/* Set error transitions in the transitions that go to error. */
 	for ( TransList::Iter trans = state->outList; trans.lte(); trans++ ) {
 		if ( trans->toState == 0 )
 			trans->actionTable.setAction( ordering, action );
 	}
-
-	/* If the outlist does not cover the alphabet then an error trans
-	 * needs to go into the default transition. */
-	if ( !outListCovers( state ) ) {
-		if ( state->outDefault == 0 ) {
-			/* There is no default transition, make one and set the
-			 * error action. */
-			TransAp *newTrans = attachNewTrans( state, 0, KeyTypeDefault, 0, 0 );
-			newTrans->actionTable.setAction( ordering, action );
-		}
-		else if ( state->outDefault->toState == 0 ) {
-			/* There is a default that goes to the error state, add 
-			 * to the action table. */
-			state->outDefault->actionTable.setAction( ordering, action );
-		}
-	}
 }
 
+
 /* Give a target state for error transitions. */
 void FsmAp::setErrorTarget( StateAp *state, StateAp *target, int *orderings, 
 			Action **actions, int nActs )
 {
+	/* Fill any gaps in the out list with an error transition. */
+	fillGaps( state );
+
 	/* Set error target in the transitions that go to error. */
 	for ( TransList::Iter trans = state->outList; trans.lte(); trans++ ) {
 		if ( trans->toState == 0 ) {
 			/* The trans goes to error, redirect it. */
-			redirectErrorTrans( trans->fromState, target, trans, KeyTypeRange );
-			trans->actionTable.setActions( orderings, actions, nActs );
-		}
-	}
-
-	/* If the outlist does not cover the alphabet then an error trans
-	 * needs to go into the default transition. */
-	if ( !outListCovers( state ) ) {
-		if ( state->outDefault == 0 ) {
-			/* There is no default transition so make one. */
-			TransAp *trans = attachNewTrans( state, target, KeyTypeDefault, 0, 0 );
-			trans->actionTable.setActions( orderings, actions, nActs );
-		}
-		else if ( state->outDefault->toState == 0 ) {
-			/* There is a default that goes to the error state, redirect it. */
-			TransAp *trans = state->outDefault;
-			redirectErrorTrans( trans->fromState, target, trans, KeyTypeDefault );
+			redirectErrorTrans( trans->fromState, target, trans );
 			trans->actionTable.setActions( orderings, actions, nActs );
 		}
 	}
@@ -359,56 +392,6 @@ void FsmAp::middleErrorAction( int ordering, Action *action, int transferPoint )
 	}
 }
 
-void FsmAp::startContext( int contextId )
-{
-	/* Make sure the start state has no other entry points. */
-	isolateStartState();
-
-	/* Add the actions. */
-	startState->contextSet.insert( contextId );
-}
-
-void FsmAp::allContext( int contextId )
-{
-	/* Insert actions in the error action table of all states. */
-	for ( StateList::Iter state = stateList; state.lte(); state++ )
-		state->contextSet.insert( contextId );
-}
-
-void FsmAp::finalContext( int contextId )
-{
-	/* Add the action to the error table of final states. */
-	for ( StateSet::Iter state = finStateSet; state.lte(); state++ )
-		(*state)->contextSet.insert( contextId );
-}
-
-void FsmAp::notStartContext( int contextId )
-{
-	for ( StateList::Iter state = stateList; state.lte(); state++ ) {
-		if ( state != startState )
-			state->contextSet.insert( contextId );
-	}
-}
-
-void FsmAp::notFinalContext( int contextId )
-{
-	for ( StateList::Iter state = stateList; state.lte(); state++ ) {
-		if ( ! state->isFinState() )
-			state->contextSet.insert( contextId );
-	}
-}
-
-void FsmAp::middleContext( int contextId )
-{
-	/* Isolate the start state in case it is reachable from in inside the
-	 * machine, in which case we don't want it set. */
-	for ( StateList::Iter state = stateList; state.lte(); state++ ) {
-		if ( state != startState && ! state->isFinState() )
-			state->contextSet.insert( contextId );
-	}
-}
-
-
 /* Set EOF actions in the start state. */
 void FsmAp::startEOFAction( int ordering, Action *action )
 {
@@ -573,10 +556,7 @@ int FsmAp::shiftStartActionOrder( int fromOrder )
 	int maxUsed = 0;
 
 	/* Walk the start state's transitions, shifting function ordering. */
-	OutIter outIt( startState );
-	for ( ; ! outIt.end(); outIt++ ) {
-		TransAp *trans = outIt.trans;
-
+	for ( TransList::Iter trans = startState->outList; trans.lte(); trans++ ) {
 		/* Walk the function data for the transition and set the keys to
 		 * increasing values starting at fromOrder. */
 		int curFromOrder = fromOrder;
@@ -600,9 +580,8 @@ void FsmAp::clearAllPriorities()
 		state->outPriorTable.empty();
 
 		/* Clear transition data from the out transitions. */
-		OutIter outIt( state );
-		for ( ; ! outIt.end(); outIt++ )
-			outIt.trans->priorTable.empty();
+		for ( TransList::Iter trans = state->outList; trans.lte(); trans++ )
+			trans->priorTable.empty();
 	}
 }
 
@@ -615,14 +594,14 @@ void FsmAp::nullActionKeys( )
 	/* For each state... */
 	for ( StateList::Iter state = stateList; state.lte(); state++ ) {
 		/* Walk the transitions for the state. */
-		for ( OutIter outIt( state ); ! outIt.end(); outIt ++ ) {
+		for ( TransList::Iter trans = state->outList; trans.lte(); trans++ ) {
 			/* Walk the action table for the transition. */
-			for ( ActionTable::Iter action = outIt.trans->actionTable;
+			for ( ActionTable::Iter action = trans->actionTable;
 					action.lte(); action++ )
 				action->key = 0;
 
 			/* Walk the action table for the transition. */
-			for ( LmActionTable::Iter action = outIt.trans->lmActionTable;
+			for ( LmActionTable::Iter action = trans->lmActionTable;
 					action.lte(); action++ )
 				action->key = 0;
 		}
@@ -672,9 +651,9 @@ void FsmAp::verifyStates()
 	}
 }
 
-/* Compare two transitions according to their relative priority.  Compare two
- * transitions according to their relative priority. Since the base transition
- * has no priority associated with it, the default is to return equal. */
+/* Compare two transitions according to their relative priority. Since the
+ * base transition has no priority associated with it, the default is to
+ * return equal. */
 int FsmAp::comparePrior( const PriorTable &priorTable1, const PriorTable &priorTable2 )
 {
 	/* Looking for differing priorities on same keys. Need to concurrently
@@ -817,3 +796,87 @@ void FsmAp::relinquishFinal( StateAp *state )
 	state->outActionTable.empty();
 	state->outPriorTable.empty();
 }
+
+/* 
+ * Setting Conditions.
+ */
+
+long FsmAp::baseKey( long key, const CondSet &condSet )
+{
+	Condition *cond = condData->condDict.find( key );
+	assert( cond != 0 );
+
+	CondSetMapEl *csMapEl = cond->condSetMap.find( condSet );
+	assert( csMapEl != 0 );
+	return csMapEl->baseKey;
+}
+
+bool FsmAp::findCondition( long key, Condition *&cond, CondSetMapEl *&csMapEl )
+{
+	for ( CondDict::Iter cdi = condData->condDict; cdi.lte(); cdi++ ) {
+		for ( CondSetMap::Iter cs = cdi->condSetMap; cs.lte(); cs++ ) {
+			int highKey = cs->baseKey + ( 1 << cs->condSet.length() );
+			if ( cs->baseKey <= key && key <= highKey ) {
+				cond = cdi;
+				csMapEl = cs;
+				return true;
+			}
+		}
+	}
+	return false;
+}
+
+CondSetMapEl *FsmAp::addCondition( long key, const CondSet &condSet )
+{
+	assert( condSet.length() > 0 );
+
+	Condition *cond = condData->condDict.find( key );
+	if ( cond == 0 ) {
+		#ifdef LOG_CONDS
+		cerr << "adding condition key = " << key << endl;
+		#endif
+		cond = new Condition( key );
+		condData->condDict.insert( cond );
+	}
+
+	CondSetMapEl *csMapEl = cond->condSetMap.find( condSet );
+	if ( csMapEl == 0 ) {
+		int numCondKeys = 1 << condSet.length();
+
+		long lowKey = condData->nextCondKey;
+		condData->nextCondKey += numCondKeys;
+
+		#ifdef LOG_CONDS
+		cerr << "adding condition set length = " << condSet.length() << 
+				" numCondKeys = " << numCondKeys << endl;
+		#endif
+		csMapEl = cond->condSetMap.insert( condSet );
+		csMapEl->baseKey = lowKey;
+	}
+
+	return csMapEl;
+}
+
+void FsmAp::setCondition( Action *condAction )
+{
+	assert( stateList.length() == 2 );
+	StateAp *state = stateList.head;
+
+	assert( state->stateCondDict.length() == 0 );
+	assert( state->outList.length() == 1 );
+	TransAp *trans = state->outList.head;
+
+	assert( trans->lowKey == trans->highKey );
+	assert( state->next->outList.length() == 0 );
+
+	long key = trans->lowKey;
+	CondSet condSet( condAction );
+
+	CondSetMapEl *csMapEl = addCondition( key, condSet );
+
+	StateCond *stateCond = new StateCond( trans->lowKey );
+	stateCond->condSet.insert( condAction );
+
+	state->stateCondDict.insert( stateCond );
+	trans->lowKey = trans->highKey = csMapEl->baseKey+1;
+}
