@@ -80,11 +80,9 @@ void FsmAp::detachFromInList( StateAp *from, StateAp *to,
 }
 
 /* Attach states on the default transition, range list or on out/in list key.
- * Type of attaching and is controlled by keyType. First makes a new
- * transition. If there is already a transition out from fromState on the
- * default, then will assertion fail. */
-TransAp *FsmAp::attachNewTrans( StateAp *from, StateAp *to, 
-		FsmKeyType keyType, long lowKey, long highKey )
+ * First makes a new transition. If there is already a transition out from
+ * fromState on the default, then will assertion fail. */
+TransAp *FsmAp::attachNewTrans( StateAp *from, StateAp *to, long lowKey, long highKey )
 {
 	/* Make the new transition. */
 	TransAp *retVal = new TransAp();
@@ -93,112 +91,59 @@ TransAp *FsmAp::attachNewTrans( StateAp *from, StateAp *to,
 	retVal->fromState = from;
 	retVal->toState = to;
 
-	switch ( keyType ) {
-	case KeyTypeRange: {
-		/* Make the entry in the out list for the transitions. */
-		from->outList.append( retVal );
+	/* Make the entry in the out list for the transitions. */
+	from->outList.append( retVal );
 
-		/* Set the the keys of the new trans. */
-		retVal->lowKey = lowKey;
-		retVal->highKey = highKey;
+	/* Set the the keys of the new trans. */
+	retVal->lowKey = lowKey;
+	retVal->highKey = highKey;
 
-		/* Attach using inRange as the head pointer. */
-		if ( to != 0 )
-			attachToInList( from, to, to->inRange.head, retVal );
-		break;
-	}
-	case KeyTypeDefault: {
-		/* If there is already a default, then assertion fail. */
-		assert( from->outDefault == 0 );
-
-		/* Attach using the new transition and defInTrans as the head. */
-		from->outDefault = retVal;
-
-		/* Keys are not used for the default transition. Init it for
-		 * consistency. */
-		retVal->lowKey = 0;
-		retVal->highKey = 0;
-
-		/* Attach using inDef as the head pointer. */
-		if ( to != 0 )
-			attachToInList( from, to, to->inDefault.head, retVal );
-		break;
-	}}
+	/* Attach using inList as the head pointer. */
+	if ( to != 0 )
+		attachToInList( from, to, to->inList.head, retVal );
 
 	return retVal;
 }
 
-/* Attach for range lists or for the default transition. Type of attaching is
- * controlled by the keyType parameter. This attach should be used when a
- * transition already is allocated and must be attached to a target state.
- * Does not handle adding the transition into the out list. */
-void FsmAp::attachTrans( StateAp *from, StateAp *to, TransAp *trans, 
-		FsmKeyType keyType )
+/* Attach for range lists or for the default transition.  This attach should
+ * be used when a transition already is allocated and must be attached to a
+ * target state.  Does not handle adding the transition into the out list. */
+void FsmAp::attachTrans( StateAp *from, StateAp *to, TransAp *trans )
 {
 	assert( trans->fromState == 0 && trans->toState == 0 );
 	trans->fromState = from;
 	trans->toState = to;
 
 	if ( to != 0 ) { 
-		switch ( keyType ) {
-		case KeyTypeRange: {
-			/* Attach using the inRange pointer as the head pointer. */
-			attachToInList( from, to, to->inRange.head, trans );
-			break;
-		}
-		case KeyTypeDefault: {
-			/* Attach using inDef as the head pointer. */
-			attachToInList( from, to, to->inDefault.head, trans );
-			break;
-		}}
+		/* Attach using the inList pointer as the head pointer. */
+		attachToInList( from, to, to->inList.head, trans );
 	}
 }
 
 /* Redirect a transition away from error and towards some state. This is just
  * like attachTrans except it requires fromState to be set and does not touch
  * it. */
-void FsmAp::redirectErrorTrans( StateAp *from, StateAp *to, TransAp *trans,
-		FsmKeyType keyType )
+void FsmAp::redirectErrorTrans( StateAp *from, StateAp *to, TransAp *trans )
 {
 	assert( trans->fromState != 0 && trans->toState == 0 );
 	trans->toState = to;
 
 	if ( to != 0 ) { 
-		switch ( keyType ) {
-		case KeyTypeRange: {
-			/* Attach using the inRange pointer as the head pointer. */
-			attachToInList( from, to, to->inRange.head, trans );
-			break;
-		}
-		case KeyTypeDefault: {
-			/* Attach using inDef as the head pointer. */
-			attachToInList( from, to, to->inDefault.head, trans );
-			break;
-		}}
+		/* Attach using the inList pointer as the head pointer. */
+		attachToInList( from, to, to->inList.head, trans );
 	}
 }
 
-/* Detach for out/in lists or for default transition. The type of detaching is
- * controlled by the keyType parameter. */
-void FsmAp::detachTrans( StateAp *from, StateAp *to, TransAp *trans, 
-		FsmKeyType keyType )
+/* Detach for out/in lists or for default transition. */
+void FsmAp::detachTrans( StateAp *from, StateAp *to, TransAp *trans )
 {
 	assert( trans->fromState == from && trans->toState == to );
 	trans->fromState = 0;
 	trans->toState = 0;
 
 	if ( to != 0 ) {
-		switch ( keyType ) {
-		case KeyTypeRange: {
-			/* Detach using to's inRange pointer as the head. */
-			detachFromInList( from, to, to->inRange.head, trans );
-			break;
-		}
-		case KeyTypeDefault: {
-			/* Detach using the to's inDefHead as the head pointer. */
-			detachFromInList( from, to, to->inDefault.head, trans );
-			break;
-		}}
+		/* Detach using to's inList pointer as the head. */
+		detachFromInList( from, to, to->inList.head, trans );
 	}
 }
 
@@ -208,46 +153,17 @@ void FsmAp::detachTrans( StateAp *from, StateAp *to, TransAp *trans,
  * state set. A detached state becomes useless and should be deleted. */
 void FsmAp::detachState( StateAp *state )
 {
-	/* Detach the in transitions from the inRange list of transitions. */
-	while ( state->inRange.head != 0 ) {
+	/* Detach the in transitions from the inList list of transitions. */
+	while ( state->inList.head != 0 ) {
 		/* Get pointers to the trans and the state. */
-		TransAp *trans = state->inRange.head;
+		TransAp *trans = state->inList.head;
 		StateAp *fromState = trans->fromState;
 
 		/* Detach the transitions from the source state. */
-		detachTrans( fromState, state, trans, KeyTypeRange );
+		detachTrans( fromState, state, trans );
 
-		/* If from has a default transition we cannot delete range
-		 * because that effectively adds the keys to the set of keys following
-		 * the default trans.  Instead null out the transition. */
-		if ( fromState->outDefault != 0 ) {
-			/* From has a default transition, we must null out the target. */
-			trans->lmActionTable.empty();
-			trans->actionTable.empty();
-			trans->priorTable.empty();
-
-			/* Attach to the error state. */
-			attachTrans( fromState, 0, trans, KeyTypeRange );
-		}
-		else {
-			/* No default, Ok to delete the transition. */
-			fromState->outList.detach( trans );
-			delete trans;
-		}
-	}
-
-	/* Detach default in transitions. */
-	while ( state->inDefault.head != 0 ) {
-		/* Get pointers to the trans and the state. */
-		TransAp *trans = state->inDefault.head;
-		StateAp *fromState = trans->fromState;
-
-		/* Detach. */
-		detachTrans( fromState, state, trans, KeyTypeDefault );
-
-		/* From state no longer has a default trans, null it out, delete the
-		 * trans. */
-		fromState->outDefault = 0;
+		/* Ok to delete the transition. */
+		fromState->outList.detach( trans );
 		delete trans;
 	}
 
@@ -258,7 +174,7 @@ void FsmAp::detachState( StateAp *state )
 	/* Detach out range transitions. */
 	for ( TransList::Iter trans = state->outList; trans.lte(); ) {
 		TransList::Iter next = trans.next();
-		detachTrans( state, trans->toState, trans, KeyTypeRange );
+		detachTrans( state, trans->toState, trans );
 		delete trans;
 		trans = next;
 	}
@@ -266,13 +182,6 @@ void FsmAp::detachState( StateAp *state )
 	/* Delete all of the out range pointers. */
 	state->outList.abandon();
 
-	/* Detach the default out transition. */
-	if ( state->outDefault != 0 ) {
-		detachTrans( state, state->outDefault->toState, state->outDefault, 
-				KeyTypeDefault );
-		delete state->outDefault;
-	}
-
 	/* Unset final stateness before detaching from graph. */
 	if ( state->stateBits & SB_ISFINAL )
 		finStateSet.remove( state );
@@ -285,14 +194,13 @@ void FsmAp::detachState( StateAp *state )
  * the from state. Used for merging a transition in to a free spot. The trans
  * can just be dropped in. It does not conflict with an existing trans and need
  * not be crossed. Returns the new transition. */
-TransAp *FsmAp::dupTrans( StateAp *from, FsmKeyType keyType, 
-		TransAp *srcTrans, bool leavingFsm )
+TransAp *FsmAp::dupTrans( StateAp *from, TransAp *srcTrans, bool leavingFsm )
 {
 	/* Make a new transition. */
 	TransAp *newTrans = new TransAp();
 
 	/* We can attach the transition, one does not exist. */
-	attachTrans( from, srcTrans->toState, newTrans, keyType );
+	attachTrans( from, srcTrans->toState, newTrans );
 		
 	/* Call the user callback to add in the original source transition. */
 	addInTrans( newTrans, srcTrans );
@@ -310,7 +218,7 @@ TransAp *FsmAp::dupTrans( StateAp *from, FsmKeyType keyType,
 
 /* In crossing, src trans and dest trans both go to existing states. Make one
  * state from the sets of states that src and dest trans go to. */
-TransAp *FsmAp::fsmAttachStates( MergeData &md, StateAp *from, FsmKeyType keyType, 
+TransAp *FsmAp::fsmAttachStates( MergeData &md, StateAp *from,
 			TransAp *destTrans, TransAp *srcTrans, bool leavingFsm )
 {
 	/* The priorities are equal. We must merge the transitions. Does the
@@ -363,10 +271,10 @@ TransAp *FsmAp::fsmAttachStates( MergeData &md, StateAp *from, FsmKeyType keyTyp
 		StateAp *targ = lastFound->targState;
 
 		/* Detach the state from existing state. */
-		detachTrans( from, existingState, destTrans, keyType );
+		detachTrans( from, existingState, destTrans );
 
 		/* Re-attach to the new target. */
-		attachTrans( from, targ, destTrans, keyType );
+		attachTrans( from, targ, destTrans );
 
 		/* Add in src trans to the existing transition that we redirected to
 		 * the new state. We don't need to merge in the from out trans data,
@@ -379,7 +287,7 @@ TransAp *FsmAp::fsmAttachStates( MergeData &md, StateAp *from, FsmKeyType keyTyp
 
 /* Two transitions are to be crossed, handle the possibility of either going
  * to the error state. */
-TransAp *FsmAp::mergeTrans( MergeData &md, StateAp *from, FsmKeyType keyType, 
+TransAp *FsmAp::mergeTrans( MergeData &md, StateAp *from,
 			TransAp *destTrans, TransAp *srcTrans, bool leavingFsm )
 {
 	TransAp *retTrans = 0;
@@ -390,8 +298,8 @@ TransAp *FsmAp::mergeTrans( MergeData &md, StateAp *from, FsmKeyType keyType,
 	}
 	else if ( destTrans->toState == 0 && srcTrans->toState != 0 ) {
 		/* Non error added into error we need to detach and reattach, */
-		detachTrans( from, destTrans->toState, destTrans, keyType );
-		attachTrans( from, srcTrans->toState, destTrans, keyType );
+		detachTrans( from, destTrans->toState, destTrans );
+		attachTrans( from, srcTrans->toState, destTrans );
 		addInTrans( destTrans, srcTrans );
 		retTrans = destTrans;
 
@@ -405,7 +313,7 @@ TransAp *FsmAp::mergeTrans( MergeData &md, StateAp *from, FsmKeyType keyType,
 	}
 	else {
 		/* Both go somewhere, run the actual cross. */
-		retTrans = fsmAttachStates( md, from, keyType, destTrans, 
+		retTrans = fsmAttachStates( md, from, destTrans, 
 				srcTrans, leavingFsm );
 
 		if ( leavingFsm )
@@ -419,7 +327,7 @@ TransAp *FsmAp::mergeTrans( MergeData &md, StateAp *from, FsmKeyType keyType,
 /* Find the trans with the higher priority. If src is lower priority then dest then
  * src is ignored. If src is higher priority than dest, then src overwrites dest. If
  * the priorities are equal, then they are merged. */
-TransAp *FsmAp::crossTransitions( MergeData &md, StateAp *from, FsmKeyType keyType, 
+TransAp *FsmAp::crossTransitions( MergeData &md, StateAp *from,
 		TransAp *destTrans, TransAp *srcTrans, bool leavingFsm )
 {
 	TransAp *retTrans;
@@ -434,8 +342,8 @@ TransAp *FsmAp::crossTransitions( MergeData &md, StateAp *from, FsmKeyType keyTy
 	if ( compareRes < 0 ) {
 		/* Src trans has a higher priority than dest, src overwrites dest.
 		 * Detach dest and return a copy of src. */
-		detachTrans( from, destTrans->toState, destTrans, keyType );
-		retTrans = dupTrans( from, keyType, srcTrans, leavingFsm );
+		detachTrans( from, destTrans->toState, destTrans );
+		retTrans = dupTrans( from, srcTrans, leavingFsm );
 	}
 	else if ( compareRes > 0 ) {
 		/* The dest trans has a higher priority, use dest. */
@@ -443,7 +351,7 @@ TransAp *FsmAp::crossTransitions( MergeData &md, StateAp *from, FsmKeyType keyTy
 	}
 	else {
 		/* Src trans and dest trans have the same priority, they must be merged. */
-		retTrans = mergeTrans( md, from, keyType, destTrans, 
+		retTrans = mergeTrans( md, from, destTrans, 
 				srcTrans, leavingFsm );
 	}
 
@@ -458,17 +366,7 @@ TransAp *FsmAp::keyInDestEl( MergeData &md, StateAp *dest, StateAp *src,
 		 NextTrans *destTel, bool leavingFsm )
 {
 	TransAp *retTrans = 0;
-
-	/* Is there a default transition? */
-	if ( src->outDefault != 0 ) {
-		/* Cross dest trans with the default from src. */
-		retTrans = crossTransitions( md, dest, KeyTypeRange, destTel->trans, 
-				src->outDefault, leavingFsm );
-	}
-	else {
-		/* No default transition from src. Just use what was in dest already. */
-		retTrans = destTel->trans;
-	}
+	retTrans = destTel->trans;
 	return retTrans;
 }
 
@@ -479,40 +377,10 @@ TransAp *FsmAp::keyInSrcEl( MergeData &md, StateAp *dest, StateAp *src,
 		NextTrans *srcTel, bool leavingFsm )
 {
 	TransAp *retTrans = 0;
-
-	if ( dest->outDefault != 0 ) {
-		/* Cross src trans with the default. Since crossing writes to the
-		 * dest, we first need a copy of the default. */
-		TransAp *newTrans = dupTrans( dest, KeyTypeRange, dest->outDefault, false );
-		retTrans = crossTransitions( md, dest, KeyTypeRange, newTrans, 
-				srcTel->trans, leavingFsm );
-	}
-	else {
-		/* Dup src's trans into the new transition element. */
-		retTrans = dupTrans( dest, KeyTypeRange, srcTel->trans, leavingFsm );
-	}
-
+	retTrans = dupTrans( dest, srcTel->trans, leavingFsm );
 	return retTrans;
 }
 
-/* Copying the default transition. */
-void FsmAp::copyDefTrans( MergeData &md, StateAp *dest, StateAp *src, bool leavingFsm )
-{
-	if ( dest->outDefault != 0 && src->outDefault != 0 ) {
-		/* Must cross the default out transitions. */
-		dest->outDefault = crossTransitions( md, dest, KeyTypeDefault,
-					dest->outDefault, src->outDefault, leavingFsm );
-	}
-	else if ( src->outDefault != 0 ) {
-		/* A default trans only in src, copy it to dest. */
-		dest->outDefault = dupTrans( dest, KeyTypeDefault,
-				src->outDefault, leavingFsm );
-	}
-	else {
-		/* Leave dest->outDefault as is. */
-	}
-}
-
 void FsmAp::changeRangeLowerKey( TransAp *trans, long oldKey, 
 		long newKey )
 {
@@ -553,7 +421,7 @@ void FsmAp::outTransCopy( MergeData &md, StateAp *dest, StateAp *src, bool leavi
 	TransList destList;
 
 	/* Set up an iterator to stop at breaks. */
-	PairIter outPair( dest, src, keyOps, true );
+	PairIter outPair( dest, src, true );
 	while ( ! outPair.end() ) {
 		switch ( outPair.userState ) {
 		case PairIter::RangeInS1: {
@@ -578,7 +446,7 @@ void FsmAp::outTransCopy( MergeData &md, StateAp *dest, StateAp *src, bool leavi
 		}
 		case PairIter::RangeOverlap: {
 			/* Exact overlap, cross them. */
-			TransAp *newTrans = crossTransitions( md, dest, KeyTypeRange,
+			TransAp *newTrans = crossTransitions( md, dest,
 				outPair.s1Tel.trans, outPair.s2Tel.trans, leavingFsm );
 
 			/* Set up the transition's keys and append to the dest list. */
@@ -592,7 +460,7 @@ void FsmAp::outTransCopy( MergeData &md, StateAp *dest, StateAp *src, bool leavi
 			 * to be coppied when it is broken. The copy goes into the first
 			 * half of the break because that is the value we have access to.
 			 * */
-			outPair.s1Tel.trans = dupTrans( dest, KeyTypeRange, 
+			outPair.s1Tel.trans = dupTrans( dest,
 					outPair.s1Tel.trans, false );
 			break;
 		}
@@ -603,25 +471,8 @@ void FsmAp::outTransCopy( MergeData &md, StateAp *dest, StateAp *src, bool leavi
 		outPair++;
 	}
 
-	if ( destList.length() == 0 || !covers || keyOps->lt( 
-			destList.tail->highKey, keyOps->highKey ) )
-	{
-		/* Dest List does not cover the alphabet, handle 
-		 * default transitions. */
-		copyDefTrans( md, dest, src, leavingFsm );
-	}
-	else if ( dest->outDefault != 0 ) {
-		/* Dest list covers the alphabet, default transition will never be
-		 * taken. */
-		detachTrans( dest, dest->outDefault->toState, 
-				dest->outDefault, KeyTypeDefault );
-		delete dest->outDefault;
-		dest->outDefault = 0;
-	}
-
-	/* Transfer destList into dest->outList. */
-	dest->outList.shallowCopy( destList );
-	destList.abandon();
+	/* Abandon the old outList and transfer destList into it. */
+	dest->outList.transfer( destList );
 }
 
 
@@ -642,25 +493,14 @@ void FsmAp::inTransMove( StateAp *dest, StateAp *src )
 	for ( EntryIdSet::Iter enId = src->entryIds; enId.lte(); enId++ )
 		changeEntry( *enId, dest, src );
 
-	/* Move the transitions in inRange. */
-	while ( src->inRange.head != 0 ) {
+	/* Move the transitions in inList. */
+	while ( src->inList.head != 0 ) {
 		/* Get trans and from state. */
-		TransAp *trans = src->inRange.head;
+		TransAp *trans = src->inList.head;
 		StateAp *fromState = trans->fromState;
 
 		/* Detach from src, reattach to dest. */
-		detachTrans( fromState, src, trans, KeyTypeRange );
-		attachTrans( fromState, dest, trans, KeyTypeRange );
-	}
-
-	/* Move the default transitions. */
-	while ( src->inDefault.head != 0 ) {
-		/* Get trans and from state. */
-		TransAp *trans = src->inDefault.head;
-		StateAp *fromState = trans->fromState;
-
-		/* Detach from src, reattach to dest. */
-		detachTrans( fromState, src, trans, KeyTypeDefault );
-		attachTrans( fromState, dest, trans, KeyTypeDefault );
+		detachTrans( fromState, src, trans );
+		attachTrans( fromState, dest, trans );
 	}
 }
