@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001, 2002 Adrian Thurston <thurston@cs.queensu.ca>
+ *  Copyright 2001, 2002, 2006 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
 /*  This file is part of Ragel.
@@ -23,6 +23,13 @@
 
 #include "fsmgraph.h"
 #include "mergesort.h"
+#include "parsedata.h"
+
+#ifdef LOG_CONDS
+#include <iostream>
+using std::cerr;
+using std::endl;
+#endif
 
 /* Make a new state. The new state will be put on the graph's
  * list of state. The new state can be created final or non final. */
@@ -57,7 +64,37 @@ void FsmAp::concatFsm( long *str, int len )
 	/* Attach subsequent states. */
 	for ( int i = 0; i < len; i++ ) {
 		StateAp *newState = addState();
-		attachNewTrans( last, newState, KeyTypeRange, str[i], str[i] );
+		attachNewTrans( last, newState, str[i], str[i] );
+		last = newState;
+	}
+
+	/* Make the last state the final state. */
+	setFinState( last );
+}
+
+/* Case insensitive version of concatFsm. */
+void FsmAp::concatFsmCI( long *str, int len )
+{
+	/* Make the first state and set it as the start state. */
+	StateAp *last = addState();
+	setStartState( last );
+
+	/* Attach subsequent states. */
+	for ( int i = 0; i < len; i++ ) {
+		StateAp *newState = addState();
+
+		if ( 'a' <= str[i] && str[i] <= 'z' ) {
+			long item = 'A' + ( str[i] - 'a' );
+			attachNewTrans( last, newState, item, item );
+		}
+
+		attachNewTrans( last, newState, str[i], str[i] );
+
+		if ( 'A' <= str[i] && str[i] <= 'Z' ) {
+			long item = 'a' + ( str[i] - 'A' );
+			attachNewTrans( last, newState, item, item );
+		}
+
 		last = newState;
 	}
 
@@ -77,7 +114,7 @@ void FsmAp::concatFsm( long chr )
 	setFinState( end );
 
 	/* Attach on the character. */
-	attachNewTrans( startState, end, KeyTypeRange, chr, chr );
+	attachNewTrans( startState, end, chr, chr );
 }
 
 /* Construct a machine that matches any character in set.  A new machine will
@@ -97,38 +134,7 @@ void FsmAp::orFsm( long *set, int len )
 
 	/* Attach on all the integers in the given string of ints. */
 	for ( int i = 0; i < len; i++ )
-		attachNewTrans( startState, end, KeyTypeRange, set[i], set[i] );
-}
-
-/* Construct a machine that matches any character.  A new machine will be made
- * with two states and a default transition between them. IsSigned determines
- * if the machine has a signed or unsigned alphabet.  Fsm operations must be
- * done on machines with the same alphabet signedness. */
-void FsmAp::dotFsm( )
-{
-	/* Two states first start, second final. */
-	setStartState( addState() );
-
-	StateAp *end = addState();
-	setFinState( end );
-
-	/* Attach on the any char. */
-	attachNewTrans( startState, end, KeyTypeDefault, 0, 0 );
-}
-
-/* Construct a machine that matches any string of characters.  A new machine
- * will be made with a single state that is both start and final and that has
- * default transition back to itself. IsSigned determines if the machine has a
- * signed or unsiged alphabet. Fsm operations must be done on machines with
- * the same signednesss. */
-void FsmAp::dotStarFsm( )
-{
-	/* One state which is final and is the start state. */
-	setStartState( addState() );
-	setFinState( startState );
-
-	/* Attach start to start on default. */
-	attachNewTrans( startState, startState, KeyTypeDefault, 0, 0 );
+		attachNewTrans( startState, end, set[i], set[i] );
 }
 
 /* Construct a machine that matches a range of characters.  A new machine will
@@ -145,7 +151,18 @@ void FsmAp::rangeFsm( long low, long high )
 	setFinState( end );
 
 	/* Attach using the range of characters. */
-	attachNewTrans( startState, end, KeyTypeRange, low, high );
+	attachNewTrans( startState, end, low, high );
+}
+
+/* Construct a machine that a repeated range of characters.  */
+void FsmAp::rangeStarFsm( long low, long high)
+{
+	/* One state which is final and is the start state. */
+	setStartState( addState() );
+	setFinState( startState );
+
+	/* Attach start to start using range of characters. */
+	attachNewTrans( startState, startState, low, high );
 }
 
 /* Construct a machine that matches the empty string.  A new machine will be
@@ -862,12 +879,275 @@ void FsmAp::isolateStartState( )
 	setMisfitAccounting( false );
 }
 
+void printCondVals( long key, long values, const CondSet &condSet )
+{
+#ifdef LOG_CONDS
+	cerr << "key " << key << " ";
+	for ( CondSet::Iter csi = condSet; csi.lte(); csi++ ) {
+		bool set = values & (1 << csi.pos());
+		if ( !set )
+			cerr << "!";
+		cerr << (*csi)->name;
+		if ( !csi.last() )
+			cerr << ", ";
+	}
+	cerr << endl;
+#endif
+}
+
+void FsmAp::makeCondTrans( StateCondDict &mergedCondDict,
+		Expansions &expansions, StateAp *state )
+{
+	CondSet emptyCondSet;
+
+	StateCondDict::Iter mergedCond = mergedCondDict;
+	TransList::Iter trans = state->outList;
+	while ( true ) {
+		if ( trans.end() || mergedCond.end() )
+			break;
+		else if ( keyOps->lt( trans->highKey, mergedCond->key ) )
+			trans.increment();
+		else if ( keyOps->lt( mergedCond->key, trans->lowKey ) )
+			mergedCond.increment();
+		else {
+			/* mergedCond->key is inside the trans. Process the overlap and skip
+			 * over the mergedCond. */
+			CondSet &mergedCS = mergedCond->condSet;
+			int numNew = 1 << mergedCS.length();
+
+			#ifdef LOG_CONDS
+			cerr << "found an expansion for base key " << mergedCond->key << endl;
+			cerr << "numNew = " << numNew << endl;
+			#endif
+
+			LongVect *expand = new LongVect( mergedCond->key );
+			expand->appendDup( 0, numNew );
+
+			long bk = baseKey( mergedCond->key, mergedCS );
+			for ( CondSet::Iter cim = mergedCS; cim.lte(); cim++ ) {
+				int bitPos = cim.pos();
+
+				for ( int n = 0; n < numNew; n++ ) {
+					bool isSet = n & (1 << bitPos);
+					if ( isSet )
+						expand->data[n+1] |= (1 << bitPos);
+				}
+			}
+
+			for ( int n = 1; n <= numNew; n++ )
+				printCondVals( bk+expand->data[n], expand->data[n], mergedCS );
+			for ( int n = 1; n <= numNew; n++ )
+				expand->data[n] += bk;
+			expansions.append( expand );
+
+			mergedCond.increment();
+		}
+	}
+
+	/* Skip to the first transition that may have a condition char. */
+	trans = state->outList;
+	while ( trans.lte() && keyOps->lteq( trans->highKey, keyOps->highKey ) )
+		trans.increment();
+
+	/* Compute the new transitions created by condition character clashes. */
+	for ( ; trans.lte(); trans++ ) {
+		Condition *cond = 0;
+		CondSetMapEl *csMapEl = 0;
+
+		long key = trans->lowKey;
+		while (true) {
+			if ( findCondition( key, cond, csMapEl ) ) {
+				#ifdef LOG_CONDS
+				cerr << "found a condition for ";
+				#endif
+				long values = key - csMapEl->baseKey;
+				printCondVals( key, values, csMapEl->condSet );
+
+				/* Get the condSet in the state. */
+				StateCond *srcCond = state->stateCondDict.find( cond->key );
+				CondSet &srcCS = srcCond != 0 ? srcCond->condSet : emptyCondSet;
+
+				/* Get the condSet in the merged state. */
+				StateCond *mergedCond = mergedCondDict.find( cond->key );
+				CondSet &mergedCS = mergedCond != 0 ? mergedCond->condSet : emptyCondSet;
+				CondSet diffCS = mergedCS;
+
+				/* Remove items that are in the srcCS from items that are
+				 * in the mergedCS. */
+				for ( CondSet::Iter ci = srcCS; ci.lte(); ci++ )
+					diffCS.remove( *ci );
+
+				if ( diffCS.length() > 0 ) {
+					#ifdef LOG_CONDS
+					cerr << "there are " << diffCS.length() << " item(s) in the "
+							"mergedCS that are not in the srcCS" << endl;
+					#endif
+					long basic = 0;
+					for ( CondSet::Iter cis = csMapEl->condSet; cis.lte(); cis++ ) {
+						if ( values & (1 << cis.pos()) ) {
+							Action **cim = mergedCS.find( *cis );
+							int bitPos = (cim - mergedCS.data);
+							basic |= 1 << bitPos;
+						}
+					}
+
+					int numNew = 1 << diffCS.length();
+					LongVect *expand = new LongVect( key );
+					expand->appendDup( basic, numNew );
+
+					long bk = baseKey( cond->key, mergedCS );
+					for ( CondSet::Iter cid = diffCS; cid.lte(); cid++ ) {
+						Action **cim = mergedCS.find( *cid );
+						int bitPos = (cim - mergedCS.data);
+						int newPos = cid.pos();
+
+						for ( int n = 0; n < numNew; n++ ) {
+							bool isSet = n & (1 << newPos);
+							if ( isSet )
+								expand->data[n+1] |= (1 << bitPos);
+						}
+					}
+
+					for ( int n = 1; n <= numNew; n++ )
+						printCondVals( bk+expand->data[n], expand->data[n], mergedCS );
+					for ( int n = 1; n <= numNew; n++ )
+						expand->data[n] += bk;
+					expansions.append( expand );
+				}
+			}
+
+			if ( keyOps->eq( key, trans->highKey ) )
+				break;
+			keyOps->inc( key );
+		}
+	}
+}
+
+
+void FsmAp::doExpand( StateAp *newState, StateAp *srcState, LongVect *expansion )
+{
+	#ifdef LOG_CONDS
+	cerr << "expansion of " << expansion->data[0] << " to";
+	for ( int to = 1; to < expansion->length(); to++ )
+		cerr << " " << expansion->data[to];
+	cerr << endl;
+	#endif
+
+	for ( int i = 2; i < expansion->length(); i++ )
+		assert( keyOps->lt( expansion->data[i-1], expansion->data[i] ) );
+
+	bool covers = false;
+	long key = expansion->data[0];
+	for ( TransList::Iter trans = srcState->outList; trans.lte(); trans++ ) {
+		if ( keyOps->eq( key, trans->lowKey ) ||
+			keyOps->lt( trans->lowKey, key ) && keyOps->lt( key, trans->highKey ) ||
+			keyOps->eq( key, trans->highKey ) )
+		{
+			for ( int to = 1; to < expansion->length(); to++ ) {
+				/* Dup src's trans into the new transition element. */
+				TransAp *newTrans = dupTrans( newState, trans, false );
+				newTrans->lowKey = newTrans->highKey = expansion->data[to];
+				appendTrans( newState->outList, newTrans, covers );
+			}
+			return;
+		}
+	}
+	assert( false );
+}
+
+void FsmAp::removeExpSource( StateAp *destState, LongVect *expansion )
+{
+	assert( misfitAccounting );
+	long key = expansion->data[0];
+
+	for ( TransList::Iter trans = destState->outList; trans.lte(); ) {
+		TransList::Iter next = trans->next;
+
+		if ( keyOps->eq( trans->lowKey, key ) && keyOps->eq( key, trans->highKey ) ) {
+			#ifdef LOG_CONDS
+			cerr << "removing exp source for key = " << key << endl;
+			#endif
+			detachTrans( trans->fromState, trans->toState, trans );
+			destState->outList.detach( trans );
+			delete trans;
+		}
+		else if ( keyOps->eq( trans->lowKey, key ) && keyOps->lt( key, trans->highKey ) ) {
+			/* Unimplemented. */
+			assert( false );
+		}
+		else if ( keyOps->lt( trans->lowKey, key ) && keyOps->eq( key, trans->highKey ) ) {
+			/* Unimplemented. */
+			assert( false );
+		}
+		else if ( keyOps->lt( trans->lowKey, key ) && keyOps->eq( key, trans->highKey ) ) {
+			/* Unimplemented. */
+			assert( false );
+		}
+		trans = next;
+	}
+}
+
 void FsmAp::mergeStates( MergeData &md, StateAp *destState, 
 		StateAp *srcState, bool leaving )
 {
+	/* Compute the stateCondDict that will go into the destination state. */
+	StateCondDict mergedCondDict( destState->stateCondDict );
+	for ( StateCondDict::Iter srcCond = srcState->stateCondDict; srcCond.lte(); srcCond++ ) {
+		StateCond *destCond = mergedCondDict.find( srcCond->key );
+		if ( destCond == 0 ) {
+			destCond = new StateCond( srcCond->key );
+			mergedCondDict.insert( destCond );
+		}
+
+		for ( CondSet::Iter ca = srcCond->condSet; ca.lte(); ca++ )
+			destCond->condSet.insert( *ca );
+	
+		addCondition( destCond->key, destCond->condSet );
+	}
+
+	/* Check for expansions in the source state. If there are expansions, then
+	 * we must make a copy of it. The source state is read-only. */
+	Expansions expns1;
+	makeCondTrans( mergedCondDict, expns1, srcState );
+
+	/* Check for expansions in the destination state. We can make these
+	 * directly. */
+	Expansions expns2;
+	makeCondTrans( mergedCondDict, expns2, destState );
+
+	/* Copy in the new state cond dict. */
+	destState->stateCondDict.empty();
+	destState->stateCondDict.transfer( mergedCondDict );
+
 	/* Get the out transitions. */
 	outTransCopy( md, destState, srcState, leaving );
 
+	if ( expns1.length() > 0 ) {
+		for ( Expansions::Iter exp = expns1; exp.lte(); exp++ ) {
+			StateAp *newState = addState();
+			doExpand( newState, srcState, *exp );
+			outTransCopy( md, destState, newState, false );
+		}
+	}
+
+	if ( expns2.length() > 0 ) {
+		for ( Expansions::Iter exp = expns2; exp.lte(); exp++ ) {
+			StateAp *newState = addState();
+			doExpand( newState, destState, *exp );
+			outTransCopy( md, destState, newState, false );
+		}
+	}
+
+	if ( expns1.length() > 0 ) {
+		for ( Expansions::Iter exp = expns1; exp.lte(); exp++ )
+			removeExpSource( destState, *exp );
+	}
+
+	if ( expns2.length() > 0 ) {
+		for ( Expansions::Iter exp = expns2; exp.lte(); exp++ )
+			removeExpSource( destState, *exp );
+	}
+
 	/* Get its bits and final state status. */
 	destState->stateBits |= ( srcState->stateBits & ~SB_ISFINAL );
 	if ( srcState->stateBits & SB_ISFINAL )
@@ -876,21 +1156,22 @@ void FsmAp::mergeStates( MergeData &md, StateAp *destState,
 	/* Draw in any properties of srcState into destState. */
 	if ( srcState == destState ) {
 		/* Duplicate the list to protect against write to source. The
-		 * priorities and context sets are not copied in because that would
-		 * have no effect. */
+		 * priorities sets are not copied in because that would have no
+		 * effect. */
 		destState->epsilonTrans.append( EpsilonTrans( srcState->epsilonTrans ) );
 
 		/* Get all actions, again duplicating to protect agains write to source. */
-		destState->toStateActionTable.setActions( ActionTable( srcState->toStateActionTable ) );
-		destState->fromStateActionTable.setActions( ActionTable( srcState->fromStateActionTable ) );
+		destState->toStateActionTable.setActions( 
+				ActionTable( srcState->toStateActionTable ) );
+		destState->fromStateActionTable.setActions( 
+				ActionTable( srcState->fromStateActionTable ) );
 		destState->outActionTable.setActions( ActionTable( srcState->outActionTable ) );
 		destState->errActionTable.setActions( ErrActionTable( srcState->errActionTable ) );
 		destState->eofActionTable.setActions( ActionTable( srcState->eofActionTable ) );
 	}
 	else {
-		/* Get the epsilons, context, out priorities. */
+		/* Get the epsilons, out priorities. */
 		destState->epsilonTrans.append( srcState->epsilonTrans );
-		destState->contextSet.insert( srcState->contextSet );
 		destState->outPriorTable.setPriors( srcState->outPriorTable );
 
 		/* Get all actions. */
