@@ -23,6 +23,7 @@
 #define _FSMGRAPH_H
 
 #include <assert.h>
+#include "common.h"
 #include "vector.h"
 #include "bstset.h"
 #include "compare.h"
@@ -33,6 +34,9 @@
 #include "sbstset.h"
 #include "sbsttable.h"
 #include "avlset.h"
+#include "avlmap.h"
+
+//#define LOG_CONDS
 
 /* Flags that control merging. */
 #define SB_GRAPH1     0x01
@@ -47,13 +51,6 @@ struct FsmAp;
 struct Action;
 struct LongestMatchPart;
 
-/* Differnt types of keys that transitions can be attached on. */
-enum FsmKeyType
-{
-	KeyTypeRange,
-	KeyTypeDefault
-};
-
 /* State list element for unambiguous access to list element. */
 struct FsmListEl 
 {
@@ -75,56 +72,7 @@ private:
 	bool *array;
 };
 
-/* An abstraction of the key operators that manages key operations such as
- * comparison and increment according the signedness of the key. */
-struct KeyOps
-{
-	/* Default to signed alphabet. */
-	KeyOps() 
-		:isAlphSigned(true) {}
-
-	/* Default to signed alphabet. */
-	KeyOps( bool isSigned ) 
-		:isAlphSigned(isSigned) {}
-
-	/* Less than. */
-	bool lt( long key1, long key2 ) {
-		return isAlphSigned ?  key1 < key2 : 
-			(unsigned long)key1 < (unsigned long)key2;
-	}
-
-	/* Greater than. */
-	bool gt( long key1, long key2 ) {
-		return isAlphSigned ? key1 > key2 : 
-			(unsigned long)key1 > (unsigned long)key2;
-	}
-
-	/* Equality. */
-	bool eq( long key1, long key2 ) {
-		return key1 == key2;
-	}
-
-	/* Decrement. Needed only for ranges. */
-	void dec( long &key ) {
-		key = isAlphSigned ? key - 1 : ((unsigned long)key)-1;
-	}
-
-	/* Increment. Needed only for ranges. */
-	void inc( long &key ) {
-		key = isAlphSigned ? key+1 : ((unsigned long)key)+1;
-	}
-
-	/* Compute the distance between two keys. */
-	unsigned long long span( long key1, long key2 ) {
-		return isAlphSigned ? 
-			(long long)key2 - (long long)key1 + 1: 
-			(unsigned long long)((unsigned long)key2) - 
-				(unsigned long long)((unsigned long)key1) + 1;
-	}
-
-	bool isAlphSigned;
-	long lowKey, highKey;
-};
+extern KeyOps *keyOps;
 
 /* Transistion Action Element. */
 typedef SBstMapEl< int, Action* > ActionTableEl;
@@ -250,7 +198,6 @@ struct PriorDesc
 {
 	int key;
 	int priority;
-	bool used;
 };
 
 /* Element in the arrays of priorities for transitions and arrays. Ordering is
@@ -449,7 +396,7 @@ struct KeySet
 	public Vector<long>
 {
 	/* Return a pointer to the item in the set on success. */
-	long *insert( long key, KeyOps *keyOps );
+	long *insert( long key );
 };
 
 struct MinPartition 
@@ -479,12 +426,88 @@ typedef Vector<EptVectEl> EptVect;
 /* Set of entry ids that go into this state. */
 typedef BstSet<int> EntryIdSet;
 
-/* Set of context ids. */
-typedef BstSet<int> ContextSet;
-
 /* Set of longest match items that may be active in a given state. */
 typedef BstSet<LongestMatchPart*> LmItemSet;
 
+/* Conditions. */
+typedef BstSet< Action*, CmpOrd<Action*> > CondSet;
+typedef Vector< long > LongVect;
+typedef Vector<LongVect*> Expansions;
+
+typedef CmpTable< Action*, CmpOrd<Action*> > CmpCondSet;
+
+struct CondKey
+{
+	CondKey( long key, Action *cond )
+		: key(key), cond(cond) {}
+
+	long key;
+	Action *cond;
+};
+
+struct CmpCondKey
+{
+	int compare( long k1, long k2 ) const
+	{
+		if ( keyOps->lt( k1, k2 ) )
+			return -1;
+		else if ( keyOps->gt( k1, k2 ) )
+			return 1;
+		return 0;
+	}
+};
+
+struct CondSetMapEl
+	: public AvlTreeEl<CondSetMapEl>
+{
+	CondSetMapEl( const CondSet &condSet )
+		: condSet(condSet), conditionId(0) {}
+	
+	const CondSet &getKey() { return condSet; }
+
+	CondSet condSet;
+	long baseKey;
+	int conditionId;
+};
+
+typedef AvlTree<CondSetMapEl, CondSet, CmpCondSet> CondSetMap;
+
+struct Condition
+	: AvlTreeEl<Condition>
+{
+	Condition( long key ) 
+		: key(key) {}
+	long getKey() { return key; }
+
+	long key;
+
+	CondSetMap condSetMap;
+};
+typedef AvlTree<Condition, long, CmpCondKey> CondDict;
+
+struct StateCond
+	: AvlTreeEl<StateCond>
+{
+	StateCond( long key ) : key(key) {}
+	long getKey() { return key; }
+
+	long key;
+
+	CondSet condSet;
+};
+typedef AvlTree<StateCond, long, CmpCondKey> StateCondDict;
+
+struct CondData
+{
+	CondData() : nextCondKey(0) {}
+
+	/* Condition info. */
+	CondDict condDict;
+	long nextCondKey;
+};
+
+extern CondData *condData;
+
 /* State class that implements actions and priorities. */
 struct StateAp 
 {
@@ -497,10 +520,9 @@ struct StateAp
 
 	/* Out transition list and the pointer for the default out trans. */
 	TransList outList;
-	TransAp *outDefault;
 
 	/* In transition Lists. */
-	TransInList inRange, inDefault;
+	TransInList inList;
 
 	/* Entry points into the state. */
 	EntryIdSet entryIds;
@@ -508,8 +530,8 @@ struct StateAp
 	/* Epsilon transitions. */
 	EpsilonTrans epsilonTrans;
 
-	/* Context set in the state. */
-	ContextSet contextSet;
+	/* Condition info. */
+	StateCondDict stateCondDict;
 
 	/* Number of in transitions from states other than ourselves. */
 	int foreignInTrans;
@@ -585,69 +607,6 @@ struct StateAp
 	LmItemSet lmItemSet;
 };
 
-/* Iterator for a state's out transitions. Objects being iterated over come
- * from two sources: out ranges and the default transition. Skips key pairs
- * that have no transition and therefore take away from the default. */
-struct OutIter
-{
-	/* Encodes the different states that an fsm iterator can be in. */
-	enum IterState {
-		Begin,
-		Range,
-		Default,
-		End
-	};
-
-	OutIter( StateAp *state );
-
-	/* Query iterator. */
-	bool lte() { return itState != End; }
-	bool end() { return itState == End; }
-	TransAp *operator++(int);
-	TransAp *operator++();
-
-	/* Iterator state data. */
-	TransAp *trans;
-	StateAp *state;
-	IterState itState;
-
-private:
-	void findNext();
-};
-
-/* Iterator for a state's in transitions. This iterator does not adhere to
- * Ragel's standard iterator interface due to the exceptional requirements.
- * Objects being iterated over come from 3 sources: in transitions, in
- * ranges and the default transitions. These points are lists of transitions. */
-struct InIter
-{
-	/* Encodes the different states that an fsm iterator can be in. */
-	enum IterState {
-		Begin,
-		Range,
-		Default,
-		End
-	};
-
-	InIter( StateAp *state );
-
-	/* Query iterator. */
-	bool lte() { return itState != End; }
-	bool end() { return itState == End; }
-	TransAp *operator++(int);
-	TransAp *operator++();
-
-	/* The trans pointer available outside this class. */
-	TransAp *trans;
-
-	/* Iterator state. */
-	StateAp *state;
-	IterState itState;
-
-private:
-	void findNext();
-};
-
 struct NextTrans
 {
 	long lowKey, highKey;
@@ -695,8 +654,7 @@ struct PairIter
 		BreakS1, BreakS2
 	};
 
-	PairIter( const StateAp *state1, const StateAp *state2, 
-			KeyOps *keyOps, bool wantBreaks );
+	PairIter( const StateAp *state1, const StateAp *state2, bool wantBreaks );
 	
 	/* Query iterator. */
 	bool lte() { return itState != End; }
@@ -713,8 +671,6 @@ struct PairIter
 	NextTrans s1Tel, s2Tel;
 	long bottomLow, bottomHigh;
 	TransAp *bottomTrans;
-
-	KeyOps *keyOps;
 	bool wantBreaks;
 
 private:
@@ -724,41 +680,28 @@ private:
 /* Compare lists of epsilon transitions. Entries are name ids of targets. */
 typedef CmpTable< int, CmpOrd<int> > CmpEpsilonTrans;
 
-/* Compare sets of context values. */
-typedef CmpTable< int, CmpOrd<int> > CmpContextSets;
-
-
 /* Compare class for the Approximate minimization. */
 class ApproxCompare
 {
 public:
-	ApproxCompare() : keyOps(0) { }
+	ApproxCompare() { }
 	int compare( const StateAp *pState1, const StateAp *pState2 );
-
-	/* Signed alphabet, used in comparison of out transitions. */
-	KeyOps *keyOps;
 };
 
 /* Compare class for the initial partitioning of a partition minimization. */
 class InitPartitionCompare
 {
 public:
-	InitPartitionCompare() : keyOps(0) { }
+	InitPartitionCompare() { }
 	int compare( const StateAp *pState1, const StateAp *pState2 );
-
-	/* Signed alphabet, used in comparison of out transitions. */
-	KeyOps *keyOps;
 };
 
 /* Compare class for the regular partitioning of a partition minimization. */
 class PartitionCompare
 {
 public:
-	PartitionCompare() : keyOps(0) { }
+	PartitionCompare() { }
 	int compare( const StateAp *pState1, const StateAp *pState2 );
-
-	/* Signed alphabet, used in comparison of out transitions. */
-	KeyOps *keyOps;
 };
 
 /* Compare class for a minimization that marks pairs. Provides the shouldMark
@@ -766,12 +709,9 @@ public:
 class MarkCompare
 {
 public:
-	MarkCompare() : keyOps(0) { }
+	MarkCompare() { }
 	bool shouldMark( MarkIndex &markIndex, const StateAp *pState1, 
 			const StateAp *pState2 );
-
-	/* Signed alphabet, used in comparison of out transitions. */
-	KeyOps *keyOps;
 };
 
 /* List of partitions. */
@@ -790,7 +730,7 @@ typedef Vector<EntryMapEl> EntryMapBase;
 struct FsmAp 
 {
 	/* Constructors/Destructors. */
-	FsmAp( KeyOps *keyOps );
+	FsmAp( );
 	FsmAp( const FsmAp &graph );
 	~FsmAp();
 
@@ -810,9 +750,6 @@ struct FsmAp
 	/* Misfit Accounting. Are misfits put on a separate list. */
 	bool misfitAccounting;
 
-	/* Pointer to description of key type. */
-	KeyOps *keyOps;
-
 	/*
 	 * Transition actions and priorities.
 	 */
@@ -827,6 +764,9 @@ struct FsmAp
 	void transferErrorActions( StateAp *state, int transferPoint );
 	void setErrorAction( StateAp *state, int ordering, Action *action );
 
+	/* Fill all spaces in a transition list with an error transition. */
+	void fillGaps( StateAp *state );
+
 	/* Similar to setErrorAction, instead gives a state to go to on error. */
 	void setErrorTarget( StateAp *state, StateAp *target, int *orderings, 
 			Action **actions, int nActs );
@@ -838,6 +778,12 @@ struct FsmAp
 	void leaveFsmAction( int ordering, Action *action );
 	void longMatchAction( int ordering, LongestMatchPart *lmPart );
 
+	/* Set conditions. */
+	long baseKey( long key, const CondSet &condSet );
+	bool findCondition( long key, Condition *&cond, CondSetMapEl *&csMapEl );
+	CondSetMapEl *addCondition( long key, const CondSet &condSet );
+	void setCondition( Action *cond );
+
 	/* Set error actions to execute. */
 	void startErrorAction( int ordering, Action *action, int transferPoint );
 	void allErrorAction( int ordering, Action *action, int transferPoint );
@@ -846,14 +792,6 @@ struct FsmAp
 	void notFinalErrorAction( int ordering, Action *action, int transferPoint );
 	void middleErrorAction( int ordering, Action *action, int transferPoint );
 
-	/* Set Context. */
-	void startContext( int contextId );
-	void allContext( int contextId );
-	void finalContext( int contextId );
-	void notStartContext( int contextId );
-	void notFinalContext( int contextId );
-	void middleContext( int contextId );
-
 	/* Set EOF actions. */
 	void startEOFAction( int ordering, Action *action );
 	void allEOFAction( int ordering, Action *action );
@@ -924,20 +862,17 @@ struct FsmAp
 	void detachFromInList( StateAp *from, StateAp *to, TransAp *&head, TransAp *trans );
 
 	/* Attach with a new transition. */
-	TransAp *attachNewTrans( StateAp *from, StateAp *to, FsmKeyType keyType, 
+	TransAp *attachNewTrans( StateAp *from, StateAp *to,
 			long onChar1, long onChar2 );
 
 	/* Attach with an existing transition that already in an out list. */
-	void attachTrans( StateAp *from, StateAp *to, TransAp *trans, 
-			FsmKeyType keyType );
+	void attachTrans( StateAp *from, StateAp *to, TransAp *trans );
 	
 	/* Redirect a transition away from error and towards some state. */
-	void redirectErrorTrans( StateAp *from, StateAp *to, TransAp *trans,
-			FsmKeyType keyType );
+	void redirectErrorTrans( StateAp *from, StateAp *to, TransAp *trans );
 
 	/* Detach a transition from a target state. */
-	void detachTrans( StateAp *from, StateAp *to, TransAp *trans, 
-			FsmKeyType keyType );
+	void detachTrans( StateAp *from, StateAp *to, TransAp *trans );
 
 	/* When a range is split the lower key changes, this fixes it. */
 	void changeRangeLowerKey( TransAp *trans, long oldKey, long newKey );
@@ -950,28 +885,27 @@ struct FsmAp
 	 */
 
 	/* Duplicate a transition that will dropin to a free spot. */
-	TransAp *dupTrans( StateAp *from, FsmKeyType keyType,
-			TransAp *srcTrans, bool leavingFsm );
+	TransAp *dupTrans( StateAp *from, TransAp *srcTrans, bool leavingFsm );
 
 	/* In crossing, src trans overwrites the existing one because it has a
 	 * higher priority. */
-	TransAp *overwriteTrans( MergeData &md, StateAp *from, FsmKeyType keyType, 
+	TransAp *overwriteTrans( MergeData &md, StateAp *from,
 			TransAp *destTrans, TransAp *srcTrans, bool leavingFsm );
 
 	/* In crossing, two transitions both go to real states. */
-	TransAp *fsmAttachStates( MergeData &md, StateAp *from, FsmKeyType keyType,
+	TransAp *fsmAttachStates( MergeData &md, StateAp *from,
 			TransAp *destTrans, TransAp *srcTrans, bool leavingFsm );
 
 	/* Two transitions are to be crossed, handle the possibility of either
 	 * going to the error state. */
-	TransAp *mergeTrans( MergeData &md, StateAp *from, FsmKeyType keyType,
+	TransAp *mergeTrans( MergeData &md, StateAp *from,
 			TransAp *destTrans, TransAp *srcTrans, bool leavingFsm );
 
 	/* Compare deterimne relative priorities of two transition tables. */
 	int comparePrior( const PriorTable &priorTable1, const PriorTable &priorTable2 );
 
 	/* Cross a src transition with one that is already occupying a spot. */
-	TransAp *crossTransitions( MergeData &md, StateAp *from, FsmKeyType keyType, 
+	TransAp *crossTransitions( MergeData &md, StateAp *from,
 			TransAp *destTrans, TransAp *srcTrans, bool leavingFsm );
 
 	/* Common to copying transition lists and ranges. */
@@ -980,13 +914,14 @@ struct FsmAp
 	TransAp *keyInSrcEl( MergeData &md, StateAp *dest, StateAp *src, 
 			NextTrans *srcTel, bool leavingFsm );
 
-	/* Copy the default transition. */
-	void copyDefTrans( MergeData &md, StateAp *dest, StateAp *src, bool leavingFsm );
-
 	void appendTrans( TransList &destList, TransAp *trans, bool &covers );
 	void outTransCopy( MergeData &md, StateAp *dest, StateAp *src, bool leavingFsm );
 
 	/* Merging states into a destination state. */
+	void doExpand( StateAp *newState, StateAp *srcState, LongVect *expansion );
+	void removeExpSource( StateAp *destState, LongVect *expansion );
+	void makeCondTrans( StateCondDict &mergedCondDict,
+			Expansions &expansions, StateAp *state );
 	void mergeStates( MergeData &md, StateAp *destState, 
 			StateAp *srcState, bool leavingFsm );
 
@@ -1048,10 +983,10 @@ struct FsmAp
 
 	void concatFsm( long c );
 	void concatFsm( long *str, int len );
+	void concatFsmCI( long *str, int len );
 	void orFsm( long *set, int len );
-	void dotFsm( );
-	void dotStarFsm( );
 	void rangeFsm( long low, long high );
+	void rangeStarFsm( long low, long high );
 	void emptyFsm( );
 	void lambdaFsm( );
 
