@@ -22,18 +22,12 @@
 #include "fsmgraph.h"
 #include "mergesort.h"
 
-#include <iostream>
-using namespace std;
-
 int FsmAp::partitionRound( StateAp **statePtrs, MinPartition *parts, int numParts )
 {
 	/* Need a mergesort object and a single partition compare. */
 	MergeSort<StateAp*, PartitionCompare> mergeSort;
 	PartitionCompare partCompare;
 
-	/* Set the signed/unsigned flags of the mergesort and compare. */
-	mergeSort.keyOps = partCompare.keyOps = keyOps;
-
 	/* For each partition. */
 	for ( int p = 0; p < numParts; p++ ) {
 		/* Fill the pointer array with the states in the partition. */
@@ -92,9 +86,6 @@ void FsmAp::minimizePartition1()
 	if ( stateList.length() == 0 )
 		return;
 
-	/* Set the signed/unsigned flag of the mergesort and compare. */
-	mergeSort.keyOps = initPartCompare.keyOps = keyOps;
-
 	/* 
 	 * First thing is to partition the states by final state status and
 	 * transition functions. This gives us an initial partitioning to work
@@ -165,9 +156,6 @@ int FsmAp::splitCandidates( StateAp **statePtrs, MinPartition *parts, int numPar
 	MergeSort<StateAp*, PartitionCompare> mergeSort;
 	PartitionCompare partCompare;
 
-	/* Set the signed/unsigned flag of the mergesort and compare. */
-	mergeSort.keyOps = partCompare.keyOps = keyOps;
-
 	/* The lists of unsplitable (partList) and splitable partitions. 
 	 * Only partitions in the splitable list are check for needing splitting. */
 	PartitionList partList, splittable;
@@ -182,12 +170,10 @@ int FsmAp::splitCandidates( StateAp **statePtrs, MinPartition *parts, int numPar
 		parts[p].active = false;
 
 		/* Look for a trans out of any state in the partition. */
-		StateList::Iter state = parts[p].list;
-		for ( ; state.lte(); state++ ) {
+		for ( StateList::Iter state = parts[p].list; state.lte(); state++ ) {
 			/* If there is at least one transition out to another state then 
 			 * the partition becomes splittable. */
-			OutIter outIt( state );
-			if ( ! outIt.end() ) {
+			if ( state->outList.length() > 0 ) {
 				parts[p].active = true;
 				break;
 			}
@@ -268,10 +254,8 @@ int FsmAp::splitCandidates( StateAp **statePtrs, MinPartition *parts, int numPar
 			for ( ; state.lte(); state++ ) {
 				/* Walk all transition into the state and put the partition
 				 * that the from state is in onto the splittable list. */
-				InIter inIt( state );
-				for ( ; !inIt.end(); inIt++ ) {
-					MinPartition *fromPart = 
-							inIt.trans->fromState->alg.partition;
+				for ( TransInList::Iter trans = state->inList; trans.lte(); trans++ ) {
+					MinPartition *fromPart = trans->fromState->alg.partition;
 					if ( ! fromPart->active ) {
 						fromPart->active = true;
 						partList.detach( fromPart );
@@ -301,9 +285,6 @@ void FsmAp::minimizePartition2()
 	MergeSort<StateAp*, InitPartitionCompare> mergeSort;
 	InitPartitionCompare initPartCompare;
 
-	/* Set the signed/unsigned flag of the mergesort and compare. */
-	mergeSort.keyOps = initPartCompare.keyOps = keyOps;
-
 	/* Nothing to do if there are no states. */
 	if ( stateList.length() == 0 )
 		return;
@@ -364,9 +345,8 @@ void FsmAp::initialMarkRound( MarkIndex &markIndex )
 	/* P and q for walking pairs. */
 	StateAp *p = stateList.head, *q;
 
-	/* Need an initial partition compare. Set the signed/unsigned flag. */
+	/* Need an initial partition compare. */
 	InitPartitionCompare initPartCompare;
-	initPartCompare.keyOps = keyOps;
 
 	/* Walk all unordered pairs of (p, q) where p != q.
 	 * The second depth of the walk stops before reaching p. This
@@ -392,9 +372,8 @@ bool FsmAp::markRound( MarkIndex &markIndex )
 	StateAp *p = stateList.head, *q;
 	bool pairWasMarked = false;
 
-	/* Need a mark comparison. Set the signed/unsigned flag. */
+	/* Need a mark comparison. */
 	MarkCompare markCompare;
-	markCompare.keyOps = keyOps;
 
 	/* Walk all unordered pairs of (p, q) where p != q.
 	 * The second depth of the walk stops before reaching p. This
@@ -456,9 +435,6 @@ bool FsmAp::minimizeRound()
 	MergeSort<StateAp*, ApproxCompare> mergeSort;
 	ApproxCompare approxCompare;
 
-	/* Set the signed/unsigned flags of the mergesort and compare. */
-	mergeSort.keyOps = approxCompare.keyOps = keyOps;
-
 	/* Fill up an array of pointers to the states. */
 	StateAp **statePtrs = new StateAp*[stateList.length()];
 	StateList::Iter state = stateList;
