@@ -88,7 +88,7 @@ bool MarkIndex::isPairMarked(int state1, int state2)
  */
 
 /* Return a pointer to the item in the set on success. */
-long *KeySet::insert( long key, KeyOps *keyOps )
+long *KeySet::insert( long key )
 {
 	long *lower, *upper;
 	int dataLen = size();
@@ -130,18 +130,16 @@ insert:
  * out out transition data and not number. */
 StateAp::StateAp()
 :
-	/* No out transitions. */
+	/* No out or in transitions. */
 	outList(),
-	outDefault(0),
-
-	/* No in transitions. */
-	inRange(),
-	inDefault(),
+	inList(),
 
 	/* No entry points, or epsilon trans. */
 	entryIds(),
 	epsilonTrans(),
-	contextSet(),
+
+	/* Conditions. */
+	stateCondDict(),
 
 	/* No transitions in from other states. */
 	foreignInTrans(0),
@@ -172,15 +170,15 @@ StateAp::StateAp(const StateAp &other)
 	/* All lists are cleared. They will be filled in when the
 	 * individual transitions are duplicated and attached. */
 	outList(),
-	outDefault(0),
-	inRange(),
-	inDefault(),
+	inList(),
 
-	/* Duplicate the entry id set, epsilon transitions and context sets. These
+	/* Duplicate the entry id set and epsilon transitions. These
 	 * are sets of integers and as such need no fixing. */
 	entryIds(other.entryIds),
 	epsilonTrans(other.epsilonTrans),
-	contextSet(other.contextSet),
+
+	/* Copy in the elements of the avl tree-based dict. */
+	stateCondDict( other.stateCondDict ),
 
 	/* No transitions in from other states. */
 	foreignInTrans(0),
@@ -210,16 +208,15 @@ StateAp::StateAp(const StateAp &other)
 		newTrans->toState = trans->toState;
 		outList.append( newTrans );
 	}
-
-	/* Duplicate the default transition if it is there. */
-	outDefault = 0;
-	if ( other.outDefault != 0 ) {
-		/* Duplicate and store the original target. */
-		outDefault = new TransAp(*other.outDefault);
-		outDefault->toState = other.outDefault->toState;
-	}
 }
 
+/* If there is a state dict element, then delete it. Everything else is left
+ * up to the FsmGraph destructor. */
+StateAp::~StateAp()
+{
+	if ( stateDictEl != 0 )
+		delete stateDictEl;
+}
 
 /* Compare two states using pointers to the states. With the approximate
  * compare the idea is that if the compare finds them the same, they can
@@ -240,38 +237,24 @@ int ApproxCompare::compare( const StateAp *state1 , const StateAp *state2 )
 	if ( compareRes != 0 )
 		return compareRes;
 	
-	/* Test context sets. */
-	compareRes = CmpContextSets::compare( state1->contextSet, 
-			state2->contextSet );
-	if ( compareRes != 0 )
-		return compareRes;
-	
 	/* Compare the out transitions. */
 	compareRes = FsmAp::compareStateData( state1, state2 );
 	if ( compareRes != 0 )
 		return compareRes;
 
-	/* Test default transitions. */
-	compareRes = FsmAp::compareFullPtr( state1->outDefault, 
-			state2->outDefault );
-	if ( compareRes != 0 )
-		return compareRes;
-
 	/* Use a pair iterator to get the transition pairs. */
-	PairIter outPair( state1, state2, keyOps, false );
+	PairIter outPair( state1, state2, false );
 	while ( ! outPair.end() ) {
 		switch ( outPair.userState ) {
 
 		case PairIter::RangeInS1:
-			compareRes = FsmAp::compareFullPtr( 
-					outPair.s1Tel.trans, state2->outDefault );
+			compareRes = FsmAp::compareFullPtr( outPair.s1Tel.trans, 0 );
 			if ( compareRes != 0 )
 				return compareRes;
 			break;
 
 		case PairIter::RangeInS2:
-			compareRes = FsmAp::compareFullPtr( 
-					state1->outDefault, outPair.s2Tel.trans );
+			compareRes = FsmAp::compareFullPtr( 0, outPair.s2Tel.trans );
 			if ( compareRes != 0 )
 				return compareRes;
 			break;
@@ -311,38 +294,24 @@ int InitPartitionCompare::compare( const StateAp *state1 , const StateAp *state2
 	if ( compareRes != 0 )
 		return compareRes;
 
-	/* Test context sets. */
-	compareRes = CmpContextSets::compare( state1->contextSet, 
-			state2->contextSet );
-	if ( compareRes != 0 )
-		return compareRes;
-
 	/* Compare the out transitions. */
 	compareRes = FsmAp::compareStateData( state1, state2 );
 	if ( compareRes != 0 )
 		return compareRes;
 
-	/* Test default transitions. */
-	compareRes = FsmAp::compareDataPtr( state1->outDefault, 
-			state2->outDefault );
-	if ( compareRes != 0 )
-		return compareRes;
-
 	/* Use a pair iterator to get the transition pairs. */
-	PairIter outPair( state1, state2, keyOps, false );
+	PairIter outPair( state1, state2, false );
 	while ( ! outPair.end() ) {
 		switch ( outPair.userState ) {
 
 		case PairIter::RangeInS1:
-			compareRes = FsmAp::compareDataPtr( 
-					outPair.s1Tel.trans, state2->outDefault );
+			compareRes = FsmAp::compareDataPtr( outPair.s1Tel.trans, 0 );
 			if ( compareRes != 0 )
 				return compareRes;
 			break;
 
 		case PairIter::RangeInS2:
-			compareRes = FsmAp::compareDataPtr( 
-					state1->outDefault, outPair.s2Tel.trans );
+			compareRes = FsmAp::compareDataPtr( 0, outPair.s2Tel.trans );
 			if ( compareRes != 0 )
 				return compareRes;
 			break;
@@ -369,26 +338,19 @@ int PartitionCompare::compare( const StateAp *state1, const StateAp *state2 )
 {
 	int compareRes;
 
-	/* Test default transitions. */
-	compareRes = FsmAp::comparePartPtr( state1->outDefault, state2->outDefault );
-	if ( compareRes != 0 )
-		return compareRes;
-
 	/* Use a pair iterator to get the transition pairs. */
-	PairIter outPair( state1, state2, keyOps, false );
+	PairIter outPair( state1, state2, false );
 	while ( ! outPair.end() ) {
 		switch ( outPair.userState ) {
 
 		case PairIter::RangeInS1:
-			compareRes = FsmAp::comparePartPtr( 
-					outPair.s1Tel.trans, state2->outDefault );
+			compareRes = FsmAp::comparePartPtr( outPair.s1Tel.trans, 0 );
 			if ( compareRes != 0 )
 				return compareRes;
 			break;
 
 		case PairIter::RangeInS2:
-			compareRes = FsmAp::comparePartPtr( 
-					state1->outDefault, outPair.s2Tel.trans );
+			compareRes = FsmAp::comparePartPtr( 0, outPair.s2Tel.trans );
 			if ( compareRes != 0 )
 				return compareRes;
 			break;
@@ -414,25 +376,18 @@ int PartitionCompare::compare( const StateAp *state1, const StateAp *state2 )
 bool MarkCompare::shouldMark( MarkIndex &markIndex, const StateAp *state1, 
 			const StateAp *state2 )
 {
-	/* Test default transitions. */
-	if ( FsmAp::shouldMarkPtr( markIndex, state1->outDefault, 
-			state2->outDefault ) )
-		return true;
-
 	/* Use a pair iterator to get the transition pairs. */
-	PairIter outPair( state1, state2, keyOps, false );
+	PairIter outPair( state1, state2, false );
 	while ( ! outPair.end() ) {
 		switch ( outPair.userState ) {
 
 		case PairIter::RangeInS1:
-			if ( FsmAp::shouldMarkPtr( markIndex, 
-					outPair.s1Tel.trans, state2->outDefault ) )
+			if ( FsmAp::shouldMarkPtr( markIndex, outPair.s1Tel.trans, 0 ) )
 				return true;
 			break;
 
 		case PairIter::RangeInS2:
-			if ( FsmAp::shouldMarkPtr( markIndex,
-					state1->outDefault, outPair.s2Tel.trans ) )
+			if ( FsmAp::shouldMarkPtr( markIndex, 0, outPair.s2Tel.trans ) )
 				return true;
 			break;
 
@@ -452,141 +407,12 @@ bool MarkCompare::shouldMark( MarkIndex &markIndex, const StateAp *state1,
 	return false;
 }
 
-
 /* Init the iterator by advancing to the first item. */
-OutIter::OutIter( StateAp *state )
-:
-	state(state),
-	itState(Begin)
-{
-	findNext();
-}
-
-/* Postfix operator. Save result for return then increment. */
-TransAp *OutIter::operator++(int)
-{
-	TransAp *retVal = trans;
-	findNext();
-	return retVal;
-}
-
-/* Prefix operator. Icrement first, then return result. */
-TransAp *OutIter::operator++()
-{
-	findNext();
-	return trans;
-}
-
-/* Advance to the next transition. When returns, trans points to the next
- * transition, unless there are no more, in which case end() returns true. */
-void OutIter::findNext()
-{
-	/* This variable is used in dummy statements that follow the entry
-	 * goto labels. The compiler needs some statement to follow the label. */
-	bool backIn;
-
-	/* Jump into the iterator routine base on the iterator state. */
-	switch ( itState ) {
-		case Begin:     goto entryBegin;
-		case Range:     goto entryRange;
-		case Default:   goto entryDefault;
-		case End:       goto entryEnd;
-	}
-
-	entryBegin:
-
-	/* Loop over out list transitions. */
-	trans = state->outList.head;
-	while ( trans != 0 ) {
-		/* Save the state of the iterator and return. */
-		CO_RETURN( Range );
-		trans = trans->next;
-	}
-
-	/* Process default transition. */
-	if ( state->outDefault != 0 ) {
-		/* Save the state of the iterator and return. */
-		trans = state->outDefault;
-		CO_RETURN( Default );
-	}
-
-	CO_RETURN( End );
-}
-
-/* Init the iterator by advancing to the first item. */
-InIter::InIter( StateAp *state )
-:
-	state(state),
-	itState(Begin)
-{
-	findNext();
-}
-
-/* Postfix operator. Save result for return then increment. */
-TransAp *InIter::operator++(int)
-{
-	TransAp *retVal = trans;
-	findNext();
-	return retVal;
-}
-
-/* Prefix operator. Icrement first, then return result. */
-TransAp *InIter::operator++()
-{
-	findNext();
-	return trans;
-}
-
-/* Advance to the next transition. When returns, trans points to the next
- * transition, unless there are no more, in which case end() returns true. */
-void InIter::findNext()
-{
-	/* This variable is used in dummy statements that follow the entry
-	 * goto labels. The compiler needs some statement to follow the label. */
-	bool backIn;
-
-	/* Jump into the iterator routine base on the iterator state. */
-	switch ( itState ) {
-		case Begin:     goto entryBegin;
-		case Range:     goto entryRange;
-		case Default:   goto entryDefault;
-		case End:       goto entryEnd;
-	}
-
-	entryBegin:
-
-	/* Loop over inRange transitions. */
-	trans = state->inRange.head;
-	while ( trans != 0 ) {
-		/* Save the state of the iterator and return. */
-		CO_RETURN( Range );
-
-		/* Next transition. */
-		trans = trans->ilnext;
-	}
-
-	/* Loop over the list of transitions at the default trans. */
-	trans = state->inDefault.head;
-	while ( trans != 0 ) {
-		/* Save the state of the iterator and return. */
-		CO_RETURN( Default );
-
-		/* Next transition. */
-		trans = trans->ilnext;
-	}
-
-	/* Done, go into end state. */
-	CO_RETURN( End );
-}
-
-/* Init the iterator by advancing to the first item. */
-PairIter::PairIter( const StateAp *state1, const StateAp *state2, 
-		KeyOps *keyOps, bool wantBreaks )
+PairIter::PairIter( const StateAp *state1, const StateAp *state2, bool wantBreaks )
 :
 	state1(state1),
 	state2(state2),
 	itState(Begin),
-	keyOps(keyOps),
 	wantBreaks(wantBreaks)
 {
 	findNext();
