@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2005 Adrian Thurston <thurston@cs.queensu.ca>
+ *  Copyright 2001-2006 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
 /*  This file is part of Ragel.
@@ -42,6 +42,25 @@ InputLoc::InputLoc( const BISON_YYLTYPE &loc )
 {
 }
 
+void TokenData::append( const TokenData &other )
+{
+	int newLength = length + other.length;
+	char *newString = new char[newLength+1];
+	memcpy( newString, data, length );
+	memcpy( newString + length, other.data, other.length );
+	newString[newLength] = 0;
+	data = newString;
+	length = newLength;
+}
+
+void TokenData::init( char c )
+{
+	data = new char[2];
+	data[0] = c;
+	data[1] = 0;
+	length = 1;
+}
+
 /* Perform minimization after an operation according 
  * to the command line args. */
 void afterOpMinimize( FsmAp *fsm )
@@ -82,8 +101,6 @@ int countTransitions( FsmAp *fsm )
 	StateAp *state = fsm->stateList.head;
 	while ( state != 0 ) {
 		numTrans += state->outList.length();
-		if ( state->outDefault != 0 )
-			numTrans += 1;
 		state = state->next;
 	}
 	return numTrans;
@@ -168,6 +185,28 @@ long makeFsmKeyHex( char *str, const InputLoc &loc, ParseData *pd )
 
 		/* Store in integer format. */
 		return (unsigned int)ul;
+	}
+
+	case AT_Long: {
+		/* Check against upper bound. */
+		if ( ul > RL_ULONG_MAX || errno == ERANGE && ul == ULONG_MAX ) {
+			error(loc) << "literal " << str << 
+					" overflows long alphabet" << endl;
+			ul = ULONG_MAX;
+		}
+
+		return (long)ul;
+	}
+
+	case AT_UnsignedLong: {
+		/* Check against upper bound. */
+		if ( ul > RL_ULONG_MAX || errno == ERANGE && ul == ULONG_MAX ) {
+			error(loc) << "literal " << str << 
+					" overflows unsigned long alphabet" << endl;
+			ul = ULONG_MAX;
+		}
+
+		return (unsigned long)ul;
 	}}
 	/* Not reached. */
 	return 0;
@@ -298,6 +337,50 @@ long makeFsmKeyDec( char *str, const InputLoc &loc, ParseData *pd )
 			ul = ULONG_MAX;
 		}
 
+		/* Store in integer format. */
+		return ul;
+	}
+
+	case AT_Long: {
+		/* Convert the number to a decimal. First reset errno so we can check
+		 * for overflow or underflow. */
+		errno = 0;
+		long l = strtol( str, 0, 10 );
+
+		/* Check lower bound. */
+		if ( l < RL_LONG_MIN || errno == ERANGE && l == LONG_MIN ) {
+	 		error(loc) << "literal " << str << 
+					" underflows long alphabet" << endl;
+			l = LONG_MIN;
+		}
+
+		/* Check upper bound. */
+		if ( l > RL_LONG_MAX || errno == ERANGE && l == LONG_MAX ) {
+			error(loc) << "literal " << str << 
+					" overflows long alphabet" << endl;
+			l = LONG_MAX;
+		}
+		return l;
+	}
+
+	case AT_UnsignedLong: {
+		/* Cannot allow negative numbers for an unsigned type. */
+		if ( str[0] == '-' ) {
+			/* Recover by skipping the dash. */
+			error(loc) << "literal " << str << 
+					" underflows unsigned long alphabet" << endl;
+			str += 1;
+		}
+
+		/* Convert to an unsigned number and bounds check it. */
+		errno = 0;
+		unsigned long ul = strtoul( str, 0, 10 );
+		if ( ul > RL_ULONG_MAX || errno == ERANGE && ul == ULONG_MAX ) {
+			error(loc) << "literal " << str << 
+					" overflows unsigned long alphabet" << endl;
+			ul = ULONG_MAX;
+		}
+
 		/* Store in integer format. */
 		return ul;
 	}}
@@ -355,34 +438,107 @@ void makeFsmKeyArray( long *result, char *data, int len, ParseData *pd )
 
 /* Like makeFsmKeyArray except the result has only unique keys. They ordering
  * will be changed. */
-int makeFsmUniqueKeyArray( long *result, char *data, int len, ParseData *pd )
+long *makeFsmUniqueKeyArray( int &resLength, char *data, int len, 
+		bool caseInsensitive, ParseData *pd )
 {
-	/* Set up a key ops for the transitions list that we will use in the
-	 * insert for the transition list that we will use to make unique keys. */
-	KeyOps keyOps( pd->isAlphSigned() );
-
 	/* Use a transitions list for getting unique keys. */
 	KeySet keySet;
 	if ( pd->isAlphSigned() ) {
 		/* Copy from a char star type. */
 		char *src = data;
 		for ( int si = 0; si < len; si++ ) {
-			/* If the key is not already in the trans list it is unique. */
-			keySet.insert( src[si], &keyOps );
+			keySet.insert( src[si] );
+			if ( caseInsensitive ) {
+				if ( 'a' <= src[si] && src[si] <= 'z' )
+					keySet.insert( 'A' + ( src[si] - 'a' ) );
+				else if ( 'A' <= src[si] && src[si] <= 'Z' )
+					keySet.insert( 'a' + ( src[si] - 'A' ) );
+			}
 		}
 	}
 	else {
 		/* Copy from an unsigned byte ptr type. */
 		unsigned char *src = (unsigned char*) data;
 		for ( int si = 0; si < len; si++ ) {
-			/* If the key is not already in the trans list it is unique. */
-			keySet.insert( src[si], &keyOps );
+			keySet.insert( src[si] );
+			if ( caseInsensitive ) {
+				if ( 'a' <= src[si] && src[si] <= 'z' )
+					keySet.insert( 'A' + ( src[si] - 'a' ) );
+				else if ( 'A' <= src[si] && src[si] <= 'Z' )
+					keySet.insert( 'a' + ( src[si] - 'A' ) );
+			}
 		}
 	}
 
 	/* Copy the data back in and return the number of unique keys written. */
+	resLength = keySet.length();
+	long *result = new long[keySet.length()];
 	memcpy( result, keySet.data, sizeof(long)*keySet.length() );
-	return keySet.length();
+	return result;
+}
+
+FsmAp *dotFsm( ParseData *pd )
+{
+	FsmAp *retFsm = new FsmAp();
+	switch ( pd->alphType ) {
+		case AT_Char:
+			retFsm->rangeFsm( RL_CHAR_MIN, RL_CHAR_MAX );
+			break;
+		case AT_UnsignedChar:
+			retFsm->rangeFsm( 0, RL_UCHAR_MAX );
+			break;
+		case AT_Short:
+			retFsm->rangeFsm( RL_SHORT_MIN, RL_SHORT_MAX );
+			break;
+		case AT_UnsignedShort:
+			retFsm->rangeFsm( 0, RL_USHORT_MAX );
+			break;
+		case AT_Int:
+			retFsm->rangeFsm( RL_INT_MIN, RL_INT_MAX );
+			break;
+		case AT_UnsignedInt:
+			retFsm->rangeFsm( 0, RL_UINT_MAX );
+			break;
+		case AT_Long:
+			retFsm->rangeFsm( RL_LONG_MIN, RL_LONG_MAX );
+			break;
+		case AT_UnsignedLong:
+			retFsm->rangeFsm( 0, RL_ULONG_MAX );
+			break;
+	}
+	return retFsm;
+}
+
+FsmAp *dotStarFsm( ParseData *pd )
+{
+	FsmAp *retFsm = new FsmAp();
+	switch ( pd->alphType ) {
+		case AT_Char:
+			retFsm->rangeStarFsm( RL_CHAR_MIN, RL_CHAR_MAX );
+			break;
+		case AT_UnsignedChar:
+			retFsm->rangeStarFsm( 0, RL_UCHAR_MAX );
+			break;
+		case AT_Short:
+			retFsm->rangeStarFsm( RL_SHORT_MIN, RL_SHORT_MAX );
+			break;
+		case AT_UnsignedShort:
+			retFsm->rangeStarFsm( 0, RL_USHORT_MAX );
+			break;
+		case AT_Int:
+			retFsm->rangeStarFsm( RL_INT_MIN, RL_INT_MAX );
+			break;
+		case AT_UnsignedInt:
+			retFsm->rangeStarFsm( 0, RL_UINT_MAX );
+			break;
+		case AT_Long:
+			retFsm->rangeStarFsm( RL_LONG_MIN, RL_LONG_MAX );
+			break;
+		case AT_UnsignedLong:
+			retFsm->rangeStarFsm( 0, RL_ULONG_MAX );
+			break;
+	}
+	return retFsm;
 }
 
 /* Make a builtin type. Depends on the signed nature of the alphabet type. */
@@ -395,13 +551,12 @@ FsmAp *makeBuiltin( BuiltinMachine builtin, ParseData *pd )
 	switch ( builtin ) {
 	case BT_Any: {
 		/* All characters. */
-		retFsm = new FsmAp( &pd->keyOps );
-		retFsm->dotFsm( );
+		retFsm = dotFsm( pd );
 		break;
 	}
 	case BT_Ascii: {
 		/* Ascii characters 0 to 127. */
-		retFsm = new FsmAp( &pd->keyOps );
+		retFsm = new FsmAp();
 		retFsm->rangeFsm( 0, 127 );
 		break;
 	}
@@ -409,18 +564,21 @@ FsmAp *makeBuiltin( BuiltinMachine builtin, ParseData *pd )
 		/* Ascii extended characters. This is the full byte range. Dependent
 		 * on signed, vs no signed. If the alphabet is one byte then just use
 		 * dot fsm. */
-		retFsm = new FsmAp( &pd->keyOps );
 		if ( pd->alphType == AT_Char || pd->alphType == AT_UnsignedChar )
-			retFsm->dotFsm( );
-		else if ( isSigned )
+			retFsm = dotFsm( pd );
+		else if ( isSigned ) {
+			retFsm = new FsmAp();
 			retFsm->rangeFsm( -128, 127 );
-		else
+		}
+		else {
+			retFsm = new FsmAp();
 			retFsm->rangeFsm( 0, 255 );
+		}
 		break;
 	}
 	case BT_Alpha: {
 		/* Alpha [A-Za-z]. */
-		FsmAp *upper = new FsmAp( &pd->keyOps ), *lower = new FsmAp( &pd->keyOps );
+		FsmAp *upper = new FsmAp(), *lower = new FsmAp();
 		upper->rangeFsm( 'A', 'Z' );
 		lower->rangeFsm( 'a', 'z' );
 		upper->unionOp( lower );
@@ -430,14 +588,14 @@ FsmAp *makeBuiltin( BuiltinMachine builtin, ParseData *pd )
 	}
 	case BT_Digit: {
 		/* Digits [0-9]. */
-		retFsm = new FsmAp( &pd->keyOps );
+		retFsm = new FsmAp();
 		retFsm->rangeFsm( '0', '9' );
 		break;
 	}
 	case BT_Alnum: {
 		/* Alpha numerics [0-9A-Za-z]. */
-		FsmAp *digit = new FsmAp( &pd->keyOps ), *lower = new FsmAp( &pd->keyOps );
-		FsmAp *upper = new FsmAp( &pd->keyOps );
+		FsmAp *digit = new FsmAp(), *lower = new FsmAp();
+		FsmAp *upper = new FsmAp();
 		digit->rangeFsm( '0', '9' );
 		upper->rangeFsm( 'A', 'Z' );
 		lower->rangeFsm( 'a', 'z' );
@@ -449,20 +607,20 @@ FsmAp *makeBuiltin( BuiltinMachine builtin, ParseData *pd )
 	}
 	case BT_Lower: {
 		/* Lower case characters. */
-		retFsm = new FsmAp( &pd->keyOps );
+		retFsm = new FsmAp();
 		retFsm->rangeFsm( 'a', 'z' );
 		break;
 	}
 	case BT_Upper: {
 		/* Upper case characters. */
-		retFsm = new FsmAp( &pd->keyOps );
+		retFsm = new FsmAp();
 		retFsm->rangeFsm( 'A', 'Z' );
 		break;
 	}
 	case BT_Cntrl: {
 		/* Control characters. */
-		FsmAp *cntrl = new FsmAp( &pd->keyOps );
-		FsmAp *highChar = new FsmAp( &pd->keyOps );
+		FsmAp *cntrl = new FsmAp();
+		FsmAp *highChar = new FsmAp();
 		cntrl->rangeFsm( 0, 31 );
 		highChar->concatFsm( 127 );
 		cntrl->unionOp( highChar );
@@ -472,22 +630,22 @@ FsmAp *makeBuiltin( BuiltinMachine builtin, ParseData *pd )
 	}
 	case BT_Graph: {
 		/* Graphical ascii characters [!-~]. */
-		retFsm = new FsmAp( &pd->keyOps );
+		retFsm = new FsmAp();
 		retFsm->rangeFsm( '!', '~' );
 		break;
 	}
 	case BT_Print: {
 		/* Printable characters. Same as graph except includes space. */
-		retFsm = new FsmAp( &pd->keyOps );
+		retFsm = new FsmAp();
 		retFsm->rangeFsm( ' ', '~' );
 		break;
 	}
 	case BT_Punct: {
 		/* Punctuation. */
-		FsmAp *range1 = new FsmAp( &pd->keyOps );
-		FsmAp *range2 = new FsmAp( &pd->keyOps );
-		FsmAp *range3 = new FsmAp( &pd->keyOps ); 
-		FsmAp *range4 = new FsmAp( &pd->keyOps );
+		FsmAp *range1 = new FsmAp();
+		FsmAp *range2 = new FsmAp();
+		FsmAp *range3 = new FsmAp(); 
+		FsmAp *range4 = new FsmAp();
 		range1->rangeFsm( '!', '/' );
 		range2->rangeFsm( ':', '@' );
 		range3->rangeFsm( '[', '`' );
@@ -501,8 +659,8 @@ FsmAp *makeBuiltin( BuiltinMachine builtin, ParseData *pd )
 	}
 	case BT_Space: {
 		/* Whitespace: [\t\v\f\n\r ]. */
-		FsmAp *cntrl = new FsmAp( &pd->keyOps );
-		FsmAp *space = new FsmAp( &pd->keyOps );
+		FsmAp *cntrl = new FsmAp();
+		FsmAp *space = new FsmAp();
 		cntrl->rangeFsm( '\t', '\r' );
 		space->concatFsm( ' ' );
 		cntrl->unionOp( space );
@@ -512,9 +670,9 @@ FsmAp *makeBuiltin( BuiltinMachine builtin, ParseData *pd )
 	}
 	case BT_Xdigit: {
 		/* Hex digits [0-9A-Fa-f]. */
-		FsmAp *digit = new FsmAp( &pd->keyOps );
-		FsmAp *upper = new FsmAp( &pd->keyOps );
-		FsmAp *lower = new FsmAp( &pd->keyOps );
+		FsmAp *digit = new FsmAp();
+		FsmAp *upper = new FsmAp();
+		FsmAp *lower = new FsmAp();
 		digit->rangeFsm( '0', '9' );
 		upper->rangeFsm( 'A', 'F' );
 		lower->rangeFsm( 'a', 'f' );
@@ -525,12 +683,12 @@ FsmAp *makeBuiltin( BuiltinMachine builtin, ParseData *pd )
 		break;
 	}
 	case BT_Lambda: {
-		retFsm = new FsmAp( &pd->keyOps );
+		retFsm = new FsmAp();
 		retFsm->lambdaFsm();
 		break;
 	}
 	case BT_Empty: {
-		retFsm = new FsmAp( &pd->keyOps );
+		retFsm = new FsmAp();
 		retFsm->emptyFsm();
 		break;
 	}}
@@ -573,7 +731,6 @@ ParseData::ParseData( char *fileName, char *fsmName,
 	getKeyExpr(0),
 	accessExpr(0),
 	curStateExpr(0),
-	nextContextId(0),
 	lowerNum(0),
 	upperNum(0),
 	fileName(fileName),
@@ -861,6 +1018,17 @@ void ParseData::resolveNameRefs( InlineList *inlineList, Action *action )
 				/* Resolve, pass action for local search. */
 				NameInst *target = resolveStateRef( *item->nameRef, item->loc, action );
 
+				/* Check if the target goes into a longest match. */
+				NameInst *search = target->parent;
+				while ( search != 0 ) {
+					if ( search->isLongestMatch ) {
+						error(item->loc) << "cannot enter inside a longest "
+								"match construction as an entry point" << endl;
+						break;
+					}
+					search = search->parent;
+				}
+
 				/* Note the reference in the name. This will cause the entry
 				 * point to survive to the end of the graph generating walk. */
 				if ( target != 0 )
@@ -901,11 +1069,16 @@ void ParseData::fillNameIndex( NameInst *from )
 		fillNameIndex( *name );
 }
 
+void ParseData::makeRootName()
+{
+	/* Create the root name. */
+	rootName = new NameInst( InputLoc(), 0, 0, nextNameId++, false );
+}
+
 /* Build the name tree and supporting data structures. */
 void ParseData::makeNameTree( GraphDictEl *dictEl )
 {
-	/* Create the root name and set up curNameInst for the walk. */
-	rootName = new NameInst( InputLoc(), 0, 0, nextNameId++, false );
+	/* Set up curNameInst for the walk. */
 	curNameInst = rootName;
 	curNameChild = 0;
 
@@ -983,6 +1156,10 @@ bool ParseData::setAlphType( char *s1, char *s2 )
 			alphType = AT_UnsignedInt;
 			valid = true;
 		}
+		else if ( strcmp( s2, "long" ) == 0 ) {
+			alphType = AT_UnsignedLong;
+			valid = true;
+		}
 	}
 	alphTypeSet = true;
 	return valid;
@@ -1004,6 +1181,10 @@ bool ParseData::setAlphType( char *s1 )
 		alphType = AT_Int;
 		valid = true;
 	}
+	else if ( strcmp( s1, "long" ) == 0 ) {
+		alphType = AT_Long;
+		valid = true;
+	}
 	alphTypeSet = true;
 	return valid;
 }
@@ -1042,6 +1223,14 @@ void ParseData::setLowerUpperRange( )
 			lowKey = RL_UINT_MIN;
 			highKey = RL_UINT_MAX;
 			break;
+		case AT_Long:
+			lowKey = RL_LONG_MIN;
+			highKey = RL_LONG_MAX;
+			break;
+		case AT_UnsignedLong:
+			lowKey = RL_ULONG_MIN;
+			highKey = RL_ULONG_MAX;
+			break;
 		}
 	}
 }
@@ -1051,9 +1240,12 @@ void ParseData::setLowerUpperRange( )
 void ParseData::initKeyOps( )
 {
 	/* Signedness and bounds. */
-	keyOps.isAlphSigned = isAlphSigned();
-	keyOps.lowKey = lowKey;
-	keyOps.highKey = highKey;
+	thisKeyOps.isAlphSigned = isAlphSigned();
+	thisKeyOps.lowKey = lowKey;
+	thisKeyOps.highKey = highKey;
+
+	thisCondData.nextCondKey = highKey;
+	keyOps->inc( thisCondData.nextCondKey );
 }
 
 void ParseData::printNameInst( NameInst *nameInst, int level )
@@ -1091,8 +1283,8 @@ void ParseData::removeActionDups( FsmAp *graph )
 	/* Loop all states. */
 	for ( StateList::Iter state = graph->stateList; state.lte(); state++ ) {
 		/* Loop all transitions. */
-		for ( OutIter outIt( state ); ! outIt.end(); outIt++ )
-			removeDups( outIt.trans->actionTable );
+		for ( TransList::Iter trans = state->outList; trans.lte(); trans++ )
+			removeDups( trans->actionTable );
 		removeDups( state->toStateActionTable );
 		removeDups( state->fromStateActionTable );
 		removeDups( state->eofActionTable );
@@ -1143,46 +1335,6 @@ void ParseData::initLongestMatchData()
 	}
 }
 
-/* Check if there is a call somewhere in the inline list. */
-bool ParseData::hasCall( InlineList *inlineList )
-{
-	for ( InlineList::Iter item = *inlineList; item.lte(); item++ ) {
-		if ( item->type == InlineItem::Call || item->type == InlineItem::CallExpr )
-			return true;
-
-		/* Need to recurse into longest match items. */
-		if ( item->type == InlineItem::LmSwitch ) {
-			LongestMatch *lm = item->longestMatch;
-			for ( LmPartList::Iter lmi = *lm->longestMatchList; lmi.lte(); lmi++ ) {
-				if ( lmi->action != 0 ) {
-					bool lmPartHasCall = hasCall( lmi->action->inlineList );
-					if ( lmPartHasCall )
-						return true;
-				}
-			}
-		}
-
-		if ( item->type == InlineItem::LmOnLast || 
-				item->type == InlineItem::LmOnNext ||
-				item->type == InlineItem::LmOnLagBehind )
-		{
-			LongestMatchPart *lmi = item->longestMatchPart;
-			if ( lmi->action != 0 ) {
-				bool lmPartHasCall = hasCall( lmi->action->inlineList );
-				if ( lmPartHasCall )
-					return true;
-			}
-		}
-
-		if ( item->children != 0 ) {
-			bool childHasCall = hasCall( item->children );
-			if ( childHasCall )
-				return true;
-		}
-	}
-	return false;
-}
-
 /* After building the graph, do some extra processing to ensure the runtime
  * data of the longest mactch operators is consistent. */
 void ParseData::setLongestMatchData( FsmAp *graph )
@@ -1203,14 +1355,15 @@ void ParseData::setLongestMatchData( FsmAp *graph )
 		 * statements. */
 		StateSet states;
 		for ( StateList::Iter state = graph->stateList; state.lte(); state++ ) {
-			for ( OutIter out( state ); out.lte(); out++ ) {
-				for ( ActionTable::Iter ati = out.trans->actionTable; ati.lte(); ati++ ) {
-					if ( out.trans->toState != 0 && hasCall( ati->value->inlineList ) )
-						states.insert( out.trans->toState );
+			for ( TransList::Iter trans = state->outList; trans.lte(); trans++ ) {
+				for ( ActionTable::Iter ati = trans->actionTable; ati.lte(); ati++ ) {
+					if ( ati->value->anyCall && trans->toState != 0 )
+						states.insert( trans->toState );
 				}
 			}
 		}
 
+
 		/* Init tokstart upon entering the above collected states. */
 		for ( StateSet::Iter ps = states; ps.lte(); ps++ ) {
 			/* This is run after duplicates are removed, we must guard against
@@ -1364,8 +1517,39 @@ FsmAp *ParseData::makeAll()
 	return mainGraph;
 }
 
+void ParseData::analyzeAction( Action *action, InlineList *inlineList )
+{
+	for ( InlineList::Iter item = *inlineList; item.lte(); item++ ) {
+		if ( item->type == InlineItem::Call || item->type == InlineItem::CallExpr )
+			action->anyCall = true;
 
-/* Check actions for bad uses of fsm directives */ 
+		/* Need to recurse into longest match items. */
+		if ( item->type == InlineItem::LmSwitch ) {
+			LongestMatch *lm = item->longestMatch;
+			for ( LmPartList::Iter lmi = *lm->longestMatchList; lmi.lte(); lmi++ ) {
+				if ( lmi->action != 0 )
+					analyzeAction( action, lmi->action->inlineList );
+			}
+		}
+
+		if ( item->type == InlineItem::LmOnLast || 
+				item->type == InlineItem::LmOnNext ||
+				item->type == InlineItem::LmOnLagBehind )
+		{
+			LongestMatchPart *lmi = item->longestMatchPart;
+			if ( lmi->action != 0 )
+				analyzeAction( action, lmi->action->inlineList );
+		}
+
+		if ( item->children != 0 )
+			analyzeAction( action, item->children );
+	}
+}
+
+
+/* Check actions for bad uses of fsm directives. We don't go inside longest
+ * match items in actions created by ragel, since we just want the user
+ * actions. */
 void ParseData::checkInlineList( Action *act, InlineList *inlineList )
 {
 	for ( InlineList::Iter item = *inlineList; item.lte(); item++ ) {
@@ -1406,20 +1590,40 @@ void ParseData::checkInlineList( Action *act, InlineList *inlineList )
 	}
 }
 
-/* Checks for bad usage of directives in action code. */
-void ParseData::checkActionCodeContents()
+void ParseData::checkAction( Action *action )
 {
-	/* Check if there are any calls in action code. */
-	for ( ActionList::Iter act = actionList; act.lte(); act++ )
-		checkInlineList( act, act->inlineList );
+	if ( action->numTransRefs > 0 || action->numToStateRefs > 0 ||
+			action->numFromStateRefs > 0 || action->numEofRefs > 0 )
+	{
+		if ( action->anyCall && !action->isLmAction ) {
+			/* Check for actions with calls that are embedded within a longest match
+			 * machine. */
+			for ( ActionRefs::Iter ar = action->actionRefs; ar.lte(); ar++ ) {
+				NameInst *check = *ar;
+				while ( check != 0 ) {
+					if ( check->isLongestMatch ) {
+						error(action->loc) << "fcall permitted only in pattern actions "
+							"of a longest match" << endl;
+						break;
+					}
+					check = check->parent;
+				}
+			}
+		}
+	}
+	checkInlineList( action, action->inlineList );
 }
 
+
 void ParseData::analyzeGraph( FsmAp *graph )
 {
+	for ( ActionList::Iter act = actionList; act.lte(); act++ )
+		analyzeAction( act, act->inlineList );
+
 	for ( StateList::Iter st = graph->stateList; st.lte(); st++ ) {
 		/* The transition list. */
-		for ( OutIter out = st.ptr; out.lte(); out++ ) {
-			for ( ActionTable::Iter at = out.trans->actionTable; at.lte(); at++ )
+		for ( TransList::Iter trans = st->outList; trans.lte(); trans++ ) {
+			for ( ActionTable::Iter at = trans->actionTable; at.lte(); at++ )
 				at->value->numTransRefs += 1;
 		}
 
@@ -1432,13 +1636,18 @@ void ParseData::analyzeGraph( FsmAp *graph )
 		for ( ActionTable::Iter at = st->eofActionTable; at.lte(); at++ )
 			at->value->numEofRefs += 1;
 	}
+
+	/* Checks for bad usage of directives in action code. */
+	for ( ActionList::Iter act = actionList; act.lte(); act++ )
+		checkAction( act );
 }
 
 void ParseData::prepareMachineGen( GraphDictEl *graphDictEl )
 {
-	/* Interpret the alphabet type. */
+	beginProcessing();
 	setLowerUpperRange();
 	initKeyOps();
+	makeRootName();
 	initLongestMatchData();
 
 	/* Make the graph, do minimization. */
@@ -1451,13 +1660,16 @@ void ParseData::prepareMachineGen( GraphDictEl *graphDictEl )
 	if ( gblErrorCount > 0 )
 		return;
 
-	setLongestMatchData( sectionGraph );
 	analyzeGraph( sectionGraph );
-	checkActionCodeContents();
+
+	/* Depends on the graph analysis. */
+	setLongestMatchData( sectionGraph );
 }
 
 void ParseData::generateXML( ostream &out )
 {
+	beginProcessing();
+
 	/* Make the generator. */
 	XMLCodeGen codeGen( fsmName, this, sectionGraph, out );
 
@@ -1488,7 +1700,7 @@ void writeMachines( std::ostream &out, std::string hostData, char *inputFileName
 		for ( SectionMap::Iter sect = sectionMap; sect.lte(); sect++ ) {
 			ParseData *pd = sect->value;
 			if ( pd->instanceList.length() > 0 )
-				sect->value->prepareMachineGen( 0 );
+				pd->prepareMachineGen( 0 );
 		}
 
 		if ( gblErrorCount == 0 ) {
@@ -1496,7 +1708,7 @@ void writeMachines( std::ostream &out, std::string hostData, char *inputFileName
 			for ( SectionMap::Iter sect = sectionMap; sect.lte(); sect++ ) {
 				ParseData *pd = sect->value;
 				if ( pd->instanceList.length() > 0 )
-					sect->value->generateXML( out );
+					pd->generateXML( out );
 			}
 			out << hostData;
 			out << "</ragel>\n";
