@@ -25,41 +25,18 @@
 #include <limits.h>
 #include "avlmap.h"
 #include "bstmap.h"
-#include "astring.h"
-#include "vectsimp.h"
+#include "vector.h"
 #include "dlist.h"
 #include "fsmgraph.h"
 #include "compare.h"
 #include "vector.h"
 #include "rlparse.h"
+#include "common.h"
 
 /* Forwards. */
 struct NameInst;
 using std::ostream;
 
-/* The minimum chars are the absolute value of the real minimum because the
- * sign is stored separately in integers read in so they are compared in the
- * positive. Each is casted to an unsigned because the data part of the number
- * is in unsigned int size. 
- */
-#define RL_CHAR_MIN    ((long)((char)CHAR_MIN))
-#define RL_CHAR_MAX    ((long)((char)CHAR_MAX))
-#define RL_UCHAR_MIN   ((unsigned long)((unsigned char)0))
-#define RL_UCHAR_MAX   ((unsigned long)((unsigned char)UCHAR_MAX))
-#define RL_SHORT_MIN   ((long)((short)SHRT_MIN))
-#define RL_SHORT_MAX   ((long)((short)SHRT_MAX))
-#define RL_USHORT_MIN  ((unsigned long)((unsigned short)0))
-#define RL_USHORT_MAX  ((unsigned long)((unsigned short)USHRT_MAX))
-#define RL_INT_MIN     ((long)((int)INT_MIN))
-#define RL_INT_MAX     ((long)((int)INT_MAX))
-#define RL_UINT_MIN    ((unsigned long)((unsigned int)0))
-#define RL_UINT_MAX    ((unsigned long)((unsigned int)UINT_MAX))
-#define RL_LONG_MIN    ((long)LONG_MIN)
-#define RL_LONG_MAX    ((long)LONG_MAX)
-#define RL_ULONG_MIN   ((unsigned long)0)
-#define RL_ULONG_MAX   ((unsigned long)LONG_MAX)
-
-
 /* Types of builtin machines. */
 enum BuiltinMachine
 {
@@ -123,6 +100,7 @@ public:
 		numToStateRefs(0),
 		numFromStateRefs(0),
 		numEofRefs(0),
+		anyCall(false),
 		isLmAction(false),
 		nameRefs(nameRefs)
 	{
@@ -147,6 +125,7 @@ public:
 	int numToStateRefs;
 	int numFromStateRefs;
 	int numEofRefs;
+	bool anyCall;
 
 	bool isLmAction;
 
@@ -189,31 +168,6 @@ struct ParserAction
 	Action *action;
 };
 
-/* Context statement, kept unique by name. */
-struct Context
-{
-	Context( char *data, int id )
-		: data(data), id(id), declared(false) {}
-
-	char *data;
-	int id;
-	bool declared;
-
-	Context *prev, *next;
-};
-typedef AvlMap<char *, Context*, CmpStr> ContextMap;
-typedef AvlMapEl<char *, Context*> ContextMapEl;
-typedef DList<Context> ContextList;
-
-struct ContextEmbed
-{
-	ContextEmbed( AugType type, int contextId )
-		: type(type), contextId(contextId) { }
-	
-	AugType type;
-	int contextId;
-};
-
 struct VarDef;
 struct Join;
 struct Expression;
@@ -264,17 +218,6 @@ typedef AvlMap<char*, int, CmpStr> PriorDict;
 typedef AvlMapEl<char*, int> LocalErrDictEl;
 typedef AvlMap<char*, int, CmpStr> LocalErrDict;
 
-/* Types of alphabet supported by Ragel. */
-enum AlphType
-{
-	AT_Char,
-	AT_UnsignedChar,
-	AT_Short,
-	AT_UnsignedShort,
-	AT_Int,
-	AT_UnsignedInt
-};
-
 /* Tree of instantiated names. */
 typedef BstMapEl<char*, NameInst*> NameMapEl;
 typedef BstMap<char*, NameInst*, CmpStr> NameMap;
@@ -285,8 +228,8 @@ typedef BstSet<NameInst*> NameSet;
 struct NameInst
 {
 	NameInst( const InputLoc &loc, NameInst *parent, char *name, int id, bool isLabel ) : 
-		loc(loc), parent(parent), name(name), id(id), isLabel(isLabel), 
-		numRefs(0), numUses(0), start(0), final(0) {}
+		loc(loc), parent(parent), name(name), id(id), isLabel(isLabel),
+		isLongestMatch(false), numRefs(0), numUses(0), start(0), final(0) {}
 
 	InputLoc loc;
 
@@ -297,6 +240,7 @@ struct NameInst
 	char *name;
 	int id;
 	bool isLabel;
+	bool isLongestMatch;
 
 	int numRefs;
 	int numUses;
@@ -354,6 +298,7 @@ struct ParseData
 	/* Make a name id in the current name instantiation scope if it is not
 	 * already there. */
 	NameInst *addNameInst( const InputLoc &loc, char *data, bool isLabel );
+	void makeRootName();
 	void makeNameTree( GraphDictEl *gdNode );
 	void fillNameIndex( NameInst *from );
 	void printNameTree();
@@ -387,10 +332,12 @@ struct ParseData
 	FsmAp *makeAll();
 
 	/* Checking the contents of actions. */
-	void ParseData::checkInlineList( Action *act, InlineList *inlineList );
-	void ParseData::checkActionCodeContents();
+	void checkAction( Action *action );
+	void checkInlineList( Action *act, InlineList *inlineList );
 
+	void analyzeAction( Action *action, InlineList *inlineList );
 	void analyzeGraph( FsmAp *graph );
+
 	void prepareMachineGen( GraphDictEl *graphDictEl );
 	void generateXML( ostream &out );
 	FsmAp *sectionGraph;
@@ -447,20 +394,11 @@ struct ParseData
 	InlineList *accessExpr;
 	InlineList *curStateExpr;
 
-	/* The set of contexts. A context goes in the context list if it is
-	 * declared. */
-	ContextMap contextMap;
-	ContextList contextList;
-	int nextContextId;
-
 	/* The alphabet range. */
 	char *lowerNum, *upperNum;
 	long lowKey, highKey;
 	InputLoc rangeLowLoc, rangeHighLoc;
 
-	/* Key operators. */
-	KeyOps keyOps;
-
 	/* The name of the file the fsm is from, and the spec name. */
 	char *fileName;
 	char *fsmName;
@@ -487,7 +425,6 @@ struct ParseData
 	/* Root of the name tree used for doing local name searches. */
 	NameInst *localNameScope;
 
-	bool hasCall( InlineList *inlineList );
 	void setLmInRetLoc( InlineList *inlineList );
 	void initLongestMatchData();
 	void setLongestMatchData( FsmAp *graph );
@@ -520,6 +457,15 @@ struct ParseData
 
 	Action *setTokEnd;
 	int setTokEndOrd;
+
+	void beginProcessing()
+	{
+		::condData = &thisCondData;
+		::keyOps = &thisKeyOps;
+	}
+
+	CondData thisCondData;
+	KeyOps thisKeyOps;
 };
 
 typedef AvlMap<char*, ParseData *, CmpStr> SectionMap;
@@ -532,8 +478,11 @@ long makeFsmKeyDec( char *str, const InputLoc &loc, ParseData *pd );
 long makeFsmKeyNum( char *str, const InputLoc &loc, ParseData *pd );
 long makeFsmKeyChar( char c, ParseData *pd );
 void makeFsmKeyArray( long *result, char *data, int len, ParseData *pd );
-int makeFsmUniqueKeyArray( long *result, char *data, int len, ParseData *pd );
+long *makeFsmUniqueKeyArray( int &resLen, char *data, int len, 
+		bool caseInsensitive, ParseData *pd );
 FsmAp *makeBuiltin( BuiltinMachine builtin, ParseData *pd );
+FsmAp *dotFsm( ParseData *pd );
+FsmAp *dotStarFsm( ParseData *pd );
 
 void errorStateLabels( const NameSet &locations );
 
