@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2004 Adrian Thurston <thurston@cs.queensu.ca>
+ *  Copyright 2001-2006 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
 /*  This file is part of Ragel.
@@ -75,6 +75,9 @@ void VarDef::makeNameTree( const InputLoc &loc, ParseData *pd )
 	NameInst *prevNameInst = pd->curNameInst;
 	pd->curNameInst = pd->addNameInst( loc, name, false );
 
+	if ( joinOrLm->type == JoinOrLm::LongestMatchType )
+		pd->curNameInst->isLongestMatch = true;
+
 	/* Recurse. */
 	joinOrLm->makeNameTree( pd );
 
@@ -229,11 +232,11 @@ void LongestMatch::resolveNameRefs( ParseData *pd )
 	pd->popNameScope( nameFrame );
 }
 
-void LongestMatch::restart( FsmAp *graph, TransAp *trans, FsmKeyType keyType )
+void LongestMatch::restart( FsmAp *graph, TransAp *trans )
 {
 	StateAp *fromState = trans->fromState;
-	graph->detachTrans( fromState, trans->toState, trans, keyType );
-	graph->attachTrans( fromState, graph->startState, trans, keyType );
+	graph->detachTrans( fromState, trans->toState, trans );
+	graph->attachTrans( fromState, graph->startState, trans );
 }
 
 void LongestMatch::runLonestMatch( ParseData *pd, FsmAp *graph )
@@ -251,16 +254,16 @@ void LongestMatch::runLonestMatch( ParseData *pd, FsmAp *graph )
 	 * This must happen on a separate pass so that on each iteration of the
 	 * next pass we have the item set entries from all lmAction tables. */
 	for ( StateList::Iter st = graph->stateList; st.lte(); st++ ) {
-		for ( OutIter outIt( st ); outIt.lte(); outIt++ ) {
-			if ( outIt.trans->lmActionTable.length() > 0 ) {
-				LmActionTableEl *lmAct = outIt.trans->lmActionTable.data;
-				StateAp *toState = outIt.trans->toState;
+		for ( TransList::Iter trans = st->outList; trans.lte(); trans++ ) {
+			if ( trans->lmActionTable.length() > 0 ) {
+				LmActionTableEl *lmAct = trans->lmActionTable.data;
+				StateAp *toState = trans->toState;
 				assert( toState );
 
 				/* Check if there are transitions out, this may be a very
 				 * close approximation? Out transitions going nowhere?
 				 * FIXME: Check. */
-				if ( toState->outList.length() > 0 || toState->outDefault != 0 ) {
+				if ( toState->outList.length() > 0 ) {
 					/* Fill the item sets. */
 					graph->markReachableFromHereStopFinal( toState );
 					for ( StateList::Iter ms = graph->stateList; ms.lte(); ms++ ) {
@@ -306,29 +309,26 @@ void LongestMatch::runLonestMatch( ParseData *pd, FsmAp *graph )
 	 * restarting to affect the searching through the graph that follows. For
 	 * now take the safe route and save the list of transitions to restart
 	 * until after all searching is done. */
-	Vector<TransAp*> restartTransRange, restartTransDefault;
+	Vector<TransAp*> restartTrans;
 
 	/* Set actions that do immediate token recognition, set the longest match part
 	 * id and set the token ending. */
 	for ( StateList::Iter st = graph->stateList; st.lte(); st++ ) {
-		for ( OutIter outIt( st ); outIt.lte(); outIt++ ) {
-			if ( outIt.trans->lmActionTable.length() > 0 ) {
-				LmActionTableEl *lmAct = outIt.trans->lmActionTable.data;
-				StateAp *toState = outIt.trans->toState;
+		for ( TransList::Iter trans = st->outList; trans.lte(); trans++ ) {
+			if ( trans->lmActionTable.length() > 0 ) {
+				LmActionTableEl *lmAct = trans->lmActionTable.data;
+				StateAp *toState = trans->toState;
 				assert( toState );
 
 				/* Check if there are transitions out, this may be a very
 				 * close approximation? Out transitions going nowhere?
 				 * FIXME: Check. */
-				if ( toState->outList.length() == 0 && toState->outDefault == 0 ) {
+				if ( toState->outList.length() == 0 ) {
 					/* Can execute the immediate action for the longest match
 					 * part. Redirect the action to the start state. */
-					outIt.trans->actionTable.setAction( lmAct->key, 
+					trans->actionTable.setAction( lmAct->key, 
 							lmAct->value->actOnLast );
-					if ( outIt.itState == OutIter::Range )
-						restartTransRange.append( outIt.trans );
-					else
-						restartTransDefault.append( outIt.trans );
+					restartTrans.append( trans );
 				}
 				else {
 					/* Look for non final states that have a non-empty item
@@ -355,13 +355,13 @@ void LongestMatch::runLonestMatch( ParseData *pd, FsmAp *graph )
 					 * because the error action that matches the token will
 					 * require it. */
 					if ( nonFinalNonEmptyItemSet || maxItemSetLength > 1 )
-						outIt.trans->actionTable.setAction( pd->setTokEndOrd, pd->setTokEnd );
+						trans->actionTable.setAction( pd->setTokEndOrd, pd->setTokEnd );
 
 					/* Some states may not know which longest match item to
 					 * execute, must set it. */
 					if ( maxItemSetLength > 1 ) {
 						/* There are transitions out, another match may come. */
-						outIt.trans->actionTable.setAction( lmAct->key, 
+						trans->actionTable.setAction( lmAct->key, 
 								lmAct->value->setActId );
 					}
 				}
@@ -371,10 +371,8 @@ void LongestMatch::runLonestMatch( ParseData *pd, FsmAp *graph )
 
 	/* Now that all graph searching is done it certainly safe set the
 	 * restarting. It may be safe above, however this must be verified. */
-	for ( Vector<TransAp*>::Iter pt = restartTransRange; pt.lte(); pt++ )
-		restart( graph, *pt, KeyTypeRange );
-	for ( Vector<TransAp*>::Iter pt = restartTransDefault; pt.lte(); pt++ )
-		restart( graph, *pt, KeyTypeDefault );
+	for ( Vector<TransAp*>::Iter pt = restartTrans; pt.lte(); pt++ )
+		restart( graph, *pt );
 
 	int lmErrActionOrd = pd->curActionOrd++;
 
@@ -621,6 +619,7 @@ Expression::~Expression()
 {
 	switch ( type ) {
 		case OrType: case IntersectType: case SubtractType:
+		case StrongSubtractType:
 			delete expression;
 			delete term;
 			break;
@@ -667,6 +666,22 @@ FsmAp *Expression::walk( ParseData *pd )
 			afterOpMinimize( rtnVal );
 			break;
 		}
+		case StrongSubtractType: {
+			/* Evaluate the expression. */
+			rtnVal = expression->walk( pd );
+
+			/* Evaluate the term and pad it with any* machines. */
+			FsmAp *rhs = dotStarFsm( pd );
+			FsmAp *termFsm = term->walk( pd );
+			FsmAp *trailAnyStar = dotStarFsm( pd );
+			rhs->concatOp( termFsm );
+			rhs->concatOp( trailAnyStar );
+
+			/* Perform subtraction. */
+			rtnVal->subtractOp( rhs );
+			afterOpMinimize( rtnVal );
+			break;
+		}
 		case TermType: {
 			/* Return result of the term. */
 			rtnVal = term->walk( pd );
@@ -688,6 +703,7 @@ void Expression::makeNameTree( ParseData *pd )
 	case OrType:
 	case IntersectType:
 	case SubtractType:
+	case StrongSubtractType:
 		expression->makeNameTree( pd );
 		term->makeNameTree( pd );
 		break;
@@ -705,6 +721,7 @@ void Expression::resolveNameRefs( ParseData *pd )
 	case OrType:
 	case IntersectType:
 	case SubtractType:
+	case StrongSubtractType:
 		expression->resolveNameRefs( pd );
 		term->resolveNameRefs( pd );
 		break;
@@ -721,6 +738,9 @@ Term::~Term()
 {
 	switch ( type ) {
 		case ConcatType:
+		case RightStartType:
+		case RightFinishType:
+		case LeftType:
 			delete term;
 			delete factorWithAug;
 			break;
@@ -745,6 +765,82 @@ FsmAp *Term::walk( ParseData *pd )
 			afterOpMinimize( rtnVal );
 			break;
 		}
+		case RightStartType: {
+			/* Evaluate the Term. */
+			rtnVal = term->walk( pd );
+
+			/* Evaluate the FactorWithRep. */
+			FsmAp *rhs = factorWithAug->walk( pd );
+
+			/* Set up the priority descriptors. The left machine gets the
+			 * lower priority where as the right get the higher start priority. */
+			priorDescs[0].key = pd->nextPriorKey++;
+			priorDescs[0].priority = 0;
+			rtnVal->allTransPrior( pd->curPriorOrd++, &priorDescs[0] );
+
+			/* The start transitions right machine get the higher priority.
+			 * Use the same unique key. */
+			priorDescs[1].key = priorDescs[0].key;
+			priorDescs[1].priority = 1;
+			rhs->startFsmPrior( pd->curPriorOrd++, &priorDescs[1] );
+
+			/* Perform concatenation. */
+			rtnVal->concatOp( rhs );
+			afterOpMinimize( rtnVal );
+			break;
+		}
+		case RightFinishType: {
+			/* Evaluate the Term. */
+			rtnVal = term->walk( pd );
+
+			/* Evaluate the FactorWithRep. */
+			FsmAp *rhs = factorWithAug->walk( pd );
+
+			/* Set up the priority descriptors. The left machine gets the
+			 * lower priority where as the finishing transitions to the right
+			 * get the higher priority. */
+			priorDescs[0].key = pd->nextPriorKey++;
+			priorDescs[0].priority = 0;
+			rtnVal->allTransPrior( pd->curPriorOrd++, &priorDescs[0] );
+
+			/* The finishing transitions of the right machine get the higher
+			 * priority. Use the same unique key. */
+			priorDescs[1].key = priorDescs[0].key;
+			priorDescs[1].priority = 1;
+			rhs->finishFsmPrior( pd->curPriorOrd++, &priorDescs[1] );
+
+			/* Perform concatenation. */
+			rtnVal->concatOp( rhs );
+			afterOpMinimize( rtnVal );
+			break;
+		}
+		case LeftType: {
+			/* Evaluate the Term. */
+			rtnVal = term->walk( pd );
+
+			/* Evaluate the FactorWithRep. */
+			FsmAp *rhs = factorWithAug->walk( pd );
+
+			/* Set up the priority descriptors. The left machine gets the
+			 * higher priority. */
+			priorDescs[0].key = pd->nextPriorKey++;
+			priorDescs[0].priority = 1;
+			rtnVal->allTransPrior( pd->curPriorOrd++, &priorDescs[0] );
+
+			/* The right machine gets the lower priority.  Since
+			 * startTransPrior might unnecessarily increase the number of
+			 * states during the state machine construction process (due to
+			 * isolation), we use allTransPrior instead, which has the same
+			 * effect. */
+			priorDescs[1].key = priorDescs[0].key;
+			priorDescs[1].priority = 0;
+			rhs->allTransPrior( pd->curPriorOrd++, &priorDescs[1] );
+
+			/* Perform concatenation. */
+			rtnVal->concatOp( rhs );
+			afterOpMinimize( rtnVal );
+			break;
+		}
 		case FactorWithAugType: {
 			rtnVal = factorWithAug->walk( pd );
 			break;
@@ -757,6 +853,9 @@ void Term::makeNameTree( ParseData *pd )
 {
 	switch ( type ) {
 	case ConcatType:
+	case RightStartType:
+	case RightFinishType:
+	case LeftType:
 		term->makeNameTree( pd );
 		factorWithAug->makeNameTree( pd );
 		break;
@@ -770,6 +869,9 @@ void Term::resolveNameRefs( ParseData *pd )
 {
 	switch ( type ) {
 	case ConcatType:
+	case RightStartType:
+	case RightFinishType:
+	case LeftType:
 		term->resolveNameRefs( pd );
 		factorWithAug->resolveNameRefs( pd );
 		break;
@@ -958,39 +1060,6 @@ void FactorWithAug::assignPriorities( FsmAp *graph, int *priorOrd )
 	}
 }
 
-/* Assign context. */
-void FactorWithAug::assignContext( FsmAp *graph )
-{
-	for ( int i = 0; i < contexts.length(); i++ ) {
-		switch ( contexts[i].type ) {
-		case at_start_context:
-			graph->startContext( contexts[i].contextId );
-			break;
-		case at_all_context:
-			graph->allContext( contexts[i].contextId );
-			break;
-		case at_final_context:
-			graph->finalContext( contexts[i].contextId );
-			break;
-		case at_not_start_context:
-			graph->middleContext( contexts[i].contextId );
-			graph->finalContext( contexts[i].contextId );
-			break;
-		case at_not_final_context:
-			graph->startContext( contexts[i].contextId );
-			graph->middleContext( contexts[i].contextId );
-			break;
-		case at_middle_context:
-			graph->middleContext( contexts[i].contextId );
-			break;
-
-		default:
-			/* Parser Prevents this case. */
-			break;
-		}
-	}
-}
-
 /* Evaluate a factor with augmentation node. */
 FsmAp *FactorWithAug::walk( ParseData *pd )
 {
@@ -1002,8 +1071,8 @@ FsmAp *FactorWithAug::walk( ParseData *pd )
 	if ( actions.length() > 0 )
 		actionOrd = new int[actions.length()];
 	
-	/* First walk the list of functions, assigning the
-	 * starting function ordering. */
+	/* First walk the list of actions, assigning order to all starting
+	 * actions. */
 	for ( int i = 0; i < actions.length(); i++ ) {
 		if ( actions[i].type == at_start || 
 				actions[i].type == at_start_gbl_error ||
@@ -1017,7 +1086,7 @@ FsmAp *FactorWithAug::walk( ParseData *pd )
 	/* Evaluate the factor with repetition. */
 	FsmAp *rtnVal = factorWithRep->walk( pd );
 
-	/* Compute the remaining function call orderings. */
+	/* Compute the remaining action orderings. */
 	for ( int i = 0; i < actions.length(); i++ ) {
 		if ( actions[i].type != at_start && 
 				actions[i].type != at_start_gbl_error &&
@@ -1049,7 +1118,6 @@ FsmAp *FactorWithAug::walk( ParseData *pd )
 			/* Init the prior descriptor for the priority setting. */
 			priorDescs[i].key = priorityAugs[i].priorKey;
 			priorDescs[i].priority = priorityAugs[i].priorValue;
-			priorDescs[i].used = false;
 		}
 	}
 
@@ -1070,9 +1138,7 @@ FsmAp *FactorWithAug::walk( ParseData *pd )
 		}
 	}
 
-	/* Assign context. */
-	assignContext( rtnVal );
-
+	/* Set entry points for labels. */
 	if ( labels.length() > 0 ) {
 		/* Pop the names. */
 		pd->resetNameScope( nameFrame );
@@ -1222,14 +1288,11 @@ FsmAp *FactorWithRep::walk( ParseData *pd )
 		 * interfere with any priorities set by the user. */
 		priorDescs[0].key = pd->nextPriorKey++;
 		priorDescs[0].priority = 1;
-		priorDescs[0].used = false;
+		retFsm->allTransPrior( pd->curPriorOrd++, &priorDescs[0] );
 
 		/* Leaveing gets priority 0. Use same unique key. */
 		priorDescs[1].key = priorDescs[0].key;
 		priorDescs[1].priority = 0;
-		priorDescs[1].used = false;
-
-		retFsm->allTransPrior( pd->curPriorOrd++, &priorDescs[0] );
 		retFsm->leaveFsmPrior( pd->curPriorOrd++, &priorDescs[1] );
 
 		/* Shift over the start action orders then do the kleene star. */
@@ -1240,7 +1303,7 @@ FsmAp *FactorWithRep::walk( ParseData *pd )
 	}
 	case OptionalType: {
 		/* Make the null fsm. */
-		FsmAp *nu = new FsmAp( &pd->keyOps );
+		FsmAp *nu = new FsmAp();
 		nu->lambdaFsm( );
 
 		/* Evaluate the FactorWithRep. */
@@ -1281,7 +1344,7 @@ FsmAp *FactorWithRep::walk( ParseData *pd )
 			warning(loc) << "exactly zero repetitions results "
 					"in the null machine" << endl;
 
-			retFsm = new FsmAp( &pd->keyOps );
+			retFsm = new FsmAp();
 			retFsm->lambdaFsm();
 		}
 		else {
@@ -1310,7 +1373,7 @@ FsmAp *FactorWithRep::walk( ParseData *pd )
 			warning(loc) << "max zero repetitions results "
 					"in the null machine" << endl;
 
-			retFsm = new FsmAp( &pd->keyOps );
+			retFsm = new FsmAp();
 			retFsm->lambdaFsm();
 		}
 		else {
@@ -1372,7 +1435,7 @@ FsmAp *FactorWithRep::walk( ParseData *pd )
 			error(loc) << "invalid range repetition" << endl;
 
 			/* Return null machine as recovery. */
-			retFsm = new FsmAp( &pd->keyOps );
+			retFsm = new FsmAp();
 			retFsm->lambdaFsm();
 		}
 		else if ( lowerRep == 0 && upperRep == 0 ) {
@@ -1381,7 +1444,7 @@ FsmAp *FactorWithRep::walk( ParseData *pd )
 			warning(loc) << "zero to zero repetitions results "
 					"in the null machine" << endl;
 
-			retFsm = new FsmAp( &pd->keyOps );
+			retFsm = new FsmAp();
 			retFsm->lambdaFsm();
 		}
 		else {
@@ -1497,8 +1560,7 @@ FsmAp *FactorWithNeg::walk( ParseData *pd )
 		FsmAp *toNegate = factorWithNeg->walk( pd );
 
 		/* Negation is subtract from dot-star. */
-		retFsm = new FsmAp( &pd->keyOps );
-		retFsm->dotStarFsm( );
+		retFsm = dotStarFsm( pd );
 		retFsm->subtractOp( toNegate );
 		afterOpMinimize( retFsm );
 		break;
@@ -1508,8 +1570,7 @@ FsmAp *FactorWithNeg::walk( ParseData *pd )
 		FsmAp *toNegate = factorWithNeg->walk( pd );
 
 		/* CharNegation is subtract from dot. */
-		retFsm = new FsmAp( &pd->keyOps );
-		retFsm->dotFsm( );
+		retFsm = dotFsm( pd );
 		retFsm->subtractOp( toNegate );
 		afterOpMinimize( retFsm );
 		break;
@@ -1587,10 +1648,10 @@ FsmAp *Factor::walk( ParseData *pd )
 		rtnVal = range->walk( pd );
 		break;
 	case OrExprType:
-		rtnVal = reItem->walk( pd );
+		rtnVal = reItem->walk( pd, 0 );
 		break;
 	case RegExprType:
-		rtnVal = regExp->walk( pd );
+		rtnVal = regExp->walk( pd, 0 );
 		break;
 	case ReferenceType:
 		rtnVal = varDef->walk( pd );
@@ -1602,6 +1663,11 @@ FsmAp *Factor::walk( ParseData *pd )
 		rtnVal = longestMatch->walk( pd );
 		break;
 	}
+
+	/* Assign Conditions. */
+	for ( int i = 0; i < conditions.length(); i++ )
+		rtnVal->setCondition( conditions[i] );
+
 	return rtnVal;
 }
 
@@ -1659,14 +1725,14 @@ FsmAp *Range::walk( ParseData *pd )
 	long highKey = upperLit->makeRangeEndPoint( pd );
 
 	/* Validate the range. */
-	if ( pd->keyOps.gt( lowKey, highKey ) ) {
+	if ( keyOps->gt( lowKey, highKey ) ) {
 		/* Recover by setting upper to lower; */
 		error(lowerLit->loc) << "lower end of range is greater then upper end" << endl;
 		highKey = lowKey;
 	}
 
 	/* Return the range now that it is validated. */
-	FsmAp *retFsm = new FsmAp( &pd->keyOps );
+	FsmAp *retFsm = new FsmAp();
 	retFsm->rangeFsm( lowKey, highKey );
 	return retFsm;
 }
@@ -1678,12 +1744,12 @@ long Literal::makeRangeEndPoint( ParseData *pd )
 	switch ( type ) {
 	case Number: {
 		/* Make the fsm key from the number. */
-		retVal = makeFsmKeyNum( str, loc, pd );
+		retVal = makeFsmKeyNum( data.data, loc, pd );
 		break;
 	}
 	case LitString: {
 		/* The string should be verified length one. */
-		makeFsmKeyArray( &retVal, str, 1, pd );
+		makeFsmKeyArray( &retVal, data.data, 1, pd );
 		break;
 	}}
 	return retVal;
@@ -1698,20 +1764,23 @@ FsmAp *Literal::walk( ParseData *pd )
 	switch ( type ) {
 	case Number: {
 		/* Make the fsm key in int format. */
-		long fsmKey = makeFsmKeyNum( str, loc, pd );
+		long fsmKey = makeFsmKeyNum( data.data, loc, pd );
 		/* Make the new machine. */
-		rtnVal = new FsmAp( &pd->keyOps );
+		rtnVal = new FsmAp();
 		rtnVal->concatFsm( fsmKey );
 		break;
 	}
 	case LitString: {
 		/* Make the array of keys in int format. */
-		long *arr = new long[strlen(str)];
-		makeFsmKeyArray( arr, str, strlen(str), pd );
+		long *arr = new long[data.length];
+		makeFsmKeyArray( arr, data.data, data.length, pd );
 
 		/* Make the new machine. */
-		rtnVal = new FsmAp( &pd->keyOps );
-		rtnVal->concatFsm( arr, strlen(str) );
+		rtnVal = new FsmAp();
+		if ( caseInsensitive )
+			rtnVal->concatFsmCI( arr, data.length );
+		else
+			rtnVal->concatFsm( arr, data.length );
 		delete[] arr;
 		break;
 	}}
@@ -1732,14 +1801,18 @@ RegExpr::~RegExpr()
 }
 
 /* Evaluate a regular expression object. */
-FsmAp *RegExpr::walk( ParseData *pd )
+FsmAp *RegExpr::walk( ParseData *pd, RegExpr *rootRegex )
 {
+	/* This is the root regex, pass down a pointer to this. */
+	if ( rootRegex == 0 )
+		rootRegex = this;
+
 	FsmAp *rtnVal = 0;
 	switch ( type ) {
 		case RecurseItem: {
 			/* Walk both items. */
-			FsmAp *fsm1 = regExp->walk( pd );
-			FsmAp *fsm2 = item->walk( pd );
+			FsmAp *fsm1 = regExp->walk( pd, rootRegex );
+			FsmAp *fsm2 = item->walk( pd, rootRegex );
 			if ( fsm1 == 0 )
 				rtnVal = fsm2;
 			else {
@@ -1771,7 +1844,7 @@ ReItem::~ReItem()
 }
 
 /* Evaluate a regular expression object. */
-FsmAp *ReItem::walk( ParseData *pd )
+FsmAp *ReItem::walk( ParseData *pd, RegExpr *rootRegex )
 {
 	/* The fsm to return, is the alphabet signed? */
 	FsmAp *rtnVal = 0;
@@ -1779,38 +1852,36 @@ FsmAp *ReItem::walk( ParseData *pd )
 	switch ( type ) {
 		case Data: {
 			/* Move the data into an integer array and make a concat fsm. */
-			long *arr = new long[str.length()];
-			makeFsmKeyArray( arr, str, str.length(), pd );
+			long *arr = new long[data.length];
+			makeFsmKeyArray( arr, data.data, data.length, pd );
 
 			/* Make the concat fsm. */
-			rtnVal = new FsmAp( &pd->keyOps );
-			rtnVal->concatFsm( arr, str.length() );
-
-			/* If the data item is stared, it should be of length one. */
-			assert( ! (str.length() > 1 && star ) );
+			rtnVal = new FsmAp();
+			if ( rootRegex != 0 && rootRegex->caseInsensitive )
+				rtnVal->concatFsmCI( arr, data.length );
+			else
+				rtnVal->concatFsm( arr, data.length );
 			delete[] arr;
 			break;
 		}
 		case Dot: {
 			/* Make the dot fsm. */
-			rtnVal = new FsmAp( &pd->keyOps );
-			rtnVal->dotFsm( );
+			rtnVal = dotFsm( pd );
 			break;
 		}
 		case OrBlock: {
 			/* Get the or block and minmize it. */
-			rtnVal = orBlock->walk( pd );
+			rtnVal = orBlock->walk( pd, rootRegex );
 			rtnVal->minimizePartition2();
 			break;
 		}
 		case NegOrBlock: {
 			/* Get the or block and minimize it. */
-			FsmAp *fsm = orBlock->walk( pd );
+			FsmAp *fsm = orBlock->walk( pd, rootRegex );
 			fsm->minimizePartition2();
 
 			/* Make a dot fsm and subtract from it. */
-			rtnVal = new FsmAp( &pd->keyOps );
-			rtnVal->dotFsm( );
+			rtnVal = dotFsm( pd );
 			rtnVal->subtractOp( fsm );
 			rtnVal->minimizePartition2();
 			break;
@@ -1845,14 +1916,14 @@ ReOrBlock::~ReOrBlock()
 
 
 /* Evaluate an or block of a regular expression. */
-FsmAp *ReOrBlock::walk( ParseData *pd )
+FsmAp *ReOrBlock::walk( ParseData *pd, RegExpr *rootRegex )
 {
 	FsmAp *rtnVal = 0;
 	switch ( type ) {
 		case RecurseItem: {
 			/* Evaluate the two fsm. */
-			FsmAp *fsm1 = orBlock->walk( pd );
-			FsmAp *fsm2 = item->walk( pd );
+			FsmAp *fsm1 = orBlock->walk( pd, rootRegex );
+			FsmAp *fsm2 = item->walk( pd, rootRegex );
 			if ( fsm1 == 0 )
 				rtnVal = fsm2;
 			else {
@@ -1870,25 +1941,26 @@ FsmAp *ReOrBlock::walk( ParseData *pd )
 }
 
 /* Evaluate an or block item of a regular expression. */
-FsmAp *ReOrItem::walk( ParseData *pd )
+FsmAp *ReOrItem::walk( ParseData *pd, RegExpr *rootRegex )
 {
 	/* The return value, is the alphabet signed? */
 	FsmAp *rtnVal = 0;
 	switch ( type ) {
 	case Data: {
 		/* Make the or machine. */
-		rtnVal = new FsmAp( &pd->keyOps );
+		rtnVal = new FsmAp();
 
 		/* Put the or data into an array of ints. Note that we find unique
 		 * keys. Duplicates are silently ignored. The alternative would be to
 		 * issue warning or an error but since we can't with [a0-9a] or 'a' |
 		 * 'a' don't bother here. */
-		long *arr = new long[str.length()];
-		int len = makeFsmUniqueKeyArray( arr, str, str.length(), pd );
+		int length;
+		long *array = makeFsmUniqueKeyArray( length, data.data, data.length, 
+			rootRegex != 0 ? rootRegex->caseInsensitive : false, pd );
 
 		/* Run the or operator. */
-		rtnVal->orFsm( arr, len );
-		delete[] arr;
+		rtnVal->orFsm( array, length );
+		delete[] array;
 		break;
 	}
 	case Range: {
@@ -1897,15 +1969,43 @@ FsmAp *ReOrItem::walk( ParseData *pd )
 		long highKey = makeFsmKeyChar( upper, pd );
 
 		/* Validate the range. */
-		if ( pd->keyOps.gt( lowKey, highKey ) ) {
+		if ( keyOps->gt( lowKey, highKey ) ) {
 			/* Recover by setting upper to lower; */
 			error(loc) << "lower end of range is greater then upper end" << endl;
 			highKey = lowKey;
 		}
 
 		/* Make the range machine. */
-		rtnVal = new FsmAp( &pd->keyOps );
+		rtnVal = new FsmAp();
 		rtnVal->rangeFsm( lowKey, highKey );
+
+		if ( rootRegex != 0 && rootRegex->caseInsensitive ) {
+			if ( keyOps->lteq( lowKey, 'Z' ) && keyOps->lteq( 'A', highKey ) ) {
+				long otherLow = keyOps->lt( lowKey, 'A' ) ? 'A' : lowKey;
+				long otherHigh = keyOps->lt( 'Z', highKey ) ? 'Z' : highKey;
+
+				otherLow = 'a' + ( otherLow - 'A' );
+				otherHigh = 'a' + ( otherHigh - 'A' );
+
+				FsmAp *otherRange = new FsmAp();
+				otherRange->rangeFsm( otherLow, otherHigh );
+				rtnVal->unionOp( otherRange );
+				rtnVal->minimizePartition2();
+			}
+			else if ( keyOps->lteq( lowKey, 'z' ) && keyOps->lteq( 'a', highKey ) ) {
+				long otherLow = keyOps->lt( lowKey, 'a' ) ? 'a' : lowKey;
+				long otherHigh = keyOps->lt( 'z', highKey ) ? 'z' : highKey;
+
+				otherLow = 'A' + ( otherLow - 'a' );
+				otherHigh = 'A' + ( otherHigh - 'a' );
+
+				FsmAp *otherRange = new FsmAp();
+				otherRange->rangeFsm( otherLow, otherHigh );
+				rtnVal->unionOp( otherRange );
+				rtnVal->minimizePartition2();
+			}
+		}
+
 		break;
 	}}
 	return rtnVal;
