@@ -61,7 +61,8 @@ FsmCodeGen::FsmCodeGen( )
 	bAnyRegNextStmt(false),
 	bAnyRegCurStateRef(false),
 	bAnyRegBreak(false),
-	bAnyLmSwitchError(false)
+	bAnyLmSwitchError(false),
+	bAnyConditions(false)
 {
 }
 
@@ -212,7 +213,7 @@ void FsmCodeGen::analyzeAction( Action *act, InlineList *inlineList )
 		}
 
 		/* Check for various things in regular actions. */
-		if ( act->numTransRefs > 0 ) {
+		if ( act->numTransRefs > 0 || act->numToStateRefs > 0 || act->numFromStateRefs > 0 ) {
 			/* Any returns in regular actions? */
 			if ( item->type == InlineItem::Ret )
 				bAnyRegActionRets = true;
@@ -307,6 +308,9 @@ void FsmCodeGen::analyzeMachine()
 		if ( st->defTrans != 0 && st->defTrans->action != 0 && 
 				st->defTrans->action->anyCurStateRef() )
 			st->bAnyRegCurStateRef = true;
+		
+		if ( st->stateConds.length() > 0 )
+			bAnyConditions = true;
 	}
 
 	/* Assign ids to actions that are referenced. */
@@ -394,6 +398,22 @@ string FsmCodeGen::ACCESS()
 	return ret.str();
 }
 
+string FsmCodeGen::GET_WIDE_KEY()
+{
+	if ( anyConditions() ) 
+		return "_widec";
+	else
+		return GET_KEY();
+}
+
+string FsmCodeGen::GET_WIDE_KEY( RedStateAp *state )
+{
+	if ( state->stateConds.length() > 0 )
+		return "_widec";
+	else
+		return GET_KEY();
+}
+
 string FsmCodeGen::GET_KEY()
 {
 	ostringstream ret;
@@ -412,11 +432,12 @@ string FsmCodeGen::GET_KEY()
 
 /* Write out level number of tabs. Makes the nested binary search nice
  * looking. */
-std::ostream &FsmCodeGen::TABS( int level )
+string FsmCodeGen::TABS( int level )
 {
+	string result;
 	while ( level-- > 0 )
-		out << "\t";
-	return out;
+		result += "\t";
+	return result;
 }
 
 /* Write out a key from the fsm code gen. Depends on wether or not the key is
@@ -433,10 +454,12 @@ string FsmCodeGen::KEY( long key )
 
 void FsmCodeGen::EXEC( ostream &ret, InlineItem *item, int targState, int inFinish )
 {
-	/* The parser gives fexec two children. */
-	ret << "{" << P() << " = (";
+	/* The parser gives fexec two children. The double brackets are for D
+	 * code. If the inline list is a single word it will get interpreted as a
+	 * C-style cast by the D compiler. */
+	ret << "{" << P() << " = ((";
 	INLINE_LIST( ret, item->children, targState, inFinish );
-	ret << ")-1;}";
+	ret << "))-1;}";
 }
 
 void FsmCodeGen::LM_SWITCH( ostream &ret, InlineItem *item, 
@@ -471,12 +494,12 @@ void FsmCodeGen::LM_SWITCH( ostream &ret, InlineItem *item,
 		"\t";
 }
 
-void FsmCodeGen::LM_SET_ACT_ID( ostream &ret, InlineItem *item )
+void FsmCodeGen::SET_ACT( ostream &ret, InlineItem *item )
 {
 	ret << ACT() << " = " << item->lmId << ";";
 }
 
-void FsmCodeGen::LM_SET_TOK_END( ostream &ret, InlineItem *item )
+void FsmCodeGen::SET_TOKEND( ostream &ret, InlineItem *item )
 {
 	/* The tokend action sets tokend. */
 	ret << TOKEND() << " = " << P();
@@ -485,22 +508,22 @@ void FsmCodeGen::LM_SET_TOK_END( ostream &ret, InlineItem *item )
 	out << ";";
 }
 
-void FsmCodeGen::LM_GET_TOK_END( ostream &ret, InlineItem *item )
+void FsmCodeGen::GET_TOKEND( ostream &ret, InlineItem *item )
 {
 	ret << TOKEND();
 }
 
-void FsmCodeGen::LM_INIT_TOK_START( ostream &ret, InlineItem *item )
+void FsmCodeGen::INIT_TOKSTART( ostream &ret, InlineItem *item )
 {
 	ret << TOKSTART() << " = " << NULL_ITEM() << ";";
 }
 
-void FsmCodeGen::LM_INIT_ACT( ostream &ret, InlineItem *item )
+void FsmCodeGen::INIT_ACT( ostream &ret, InlineItem *item )
 {
 	ret << ACT() << " = 0;";
 }
 
-void FsmCodeGen::LM_SET_TOK_START( ostream &ret, InlineItem *item )
+void FsmCodeGen::SET_TOKSTART( ostream &ret, InlineItem *item )
 {
 	ret << TOKSTART() << " = " << P() << ";";
 }
@@ -548,6 +571,9 @@ void FsmCodeGen::INLINE_LIST( ostream &ret, InlineList *inlineList,
 		case InlineItem::Hold:
 			ret << P() << "--;";
 			break;
+		case InlineItem::Exec:
+			EXEC( ret, item, targState, inFinish );
+			break;
 		case InlineItem::Curs:
 			CURS( ret, inFinish );
 			break;
@@ -566,29 +592,26 @@ void FsmCodeGen::INLINE_LIST( ostream &ret, InlineList *inlineList,
 		case InlineItem::NextExpr:
 			NEXT_EXPR( ret, item, inFinish );
 			break;
-		case InlineItem::Exec:
-			EXEC( ret, item, targState, inFinish );
-			break;
 		case InlineItem::LmSwitch:
 			LM_SWITCH( ret, item, targState, inFinish );
 			break;
 		case InlineItem::LmSetActId:
-			LM_SET_ACT_ID( ret, item );
+			SET_ACT( ret, item );
 			break;
 		case InlineItem::LmSetTokEnd:
-			LM_SET_TOK_END( ret, item );
+			SET_TOKEND( ret, item );
 			break;
 		case InlineItem::LmGetTokEnd:
-			LM_GET_TOK_END( ret, item );
+			GET_TOKEND( ret, item );
 			break;
 		case InlineItem::LmInitTokStart:
-			LM_INIT_TOK_START( ret, item );
+			INIT_TOKSTART( ret, item );
 			break;
 		case InlineItem::LmInitAct:
-			LM_INIT_ACT( ret, item );
+			INIT_ACT( ret, item );
 			break;
 		case InlineItem::LmSetTokStart:
-			LM_SET_TOK_START( ret, item );
+			SET_TOKSTART( ret, item );
 			break;
 		case InlineItem::SubAction:
 			SUB_ACTION( ret, item, targState, inFinish );
@@ -623,6 +646,13 @@ void FsmCodeGen::ACTION( ostream &ret, Action *action, int targState, bool inFin
 	ret << "}\n";
 }
 
+void FsmCodeGen::CONDITION( ostream &ret, Action *condition )
+{
+	ret << "\n";
+	lineDirective( ret, cgd->fileName, condition->loc.line );
+	INLINE_LIST( ret, condition->inlineList, 0, false );
+}
+
 std::string FsmCodeGen::ERROR_STATE()
 {
 	ostringstream ret;
@@ -668,6 +698,54 @@ string FsmCodeGen::DATA_PREFIX()
 	return "";
 }
 
+void FsmCodeGen::COND_TRANSLATE( RedStateAp *state, bool genDefault )
+{
+	out << "		switch ( " << GET_KEY() << " ) {\n";
+	for ( CondVect::Iter sc = state->stateConds; sc.lte(); sc++ ) {
+		Condition *condition = *sc;
+		out << 
+			TABS(2) << "case " << condition->key << ":\n" <<
+			TABS(3) << "_widec = " << condition->baseKey << ";\n";
+
+		for ( CondSet::Iter csi = condition->condSet; csi.lte(); csi++ ) {
+			out << TABS(3) << "if ( ";
+			CONDITION( out, *csi );
+			out << " ) _widec += " << ( 1 << csi.pos() ) << ";\n";
+		}
+
+		out <<
+			TABS(3) << "break;\n";
+	}
+
+	if ( genDefault ) {
+		out << 
+			TABS(2) << "default:\n" <<
+			TABS(3) << "_widec = " << GET_KEY() << ";\n" <<
+			TABS(3) << "break;\n";
+	}
+		
+	out <<
+		"		}\n";
+}
+
+void FsmCodeGen::COND_TRANSLATE()
+{
+	out << 
+		"	_widec = " << GET_KEY() << ";\n"
+		"	switch( " << CS() << " ) {\n";
+
+	for ( RedStateList::Iter st = redFsm->stateList; st.lte(); st++ ) {
+		if ( st->stateConds.length() > 0 ) {
+			out << "	case " << st->id << ":\n";
+			COND_TRANSLATE( st, false );
+			out << "		break;\n";
+		}
+	}
+
+	out <<
+		"	}\n";
+}
+
 /*
  * Language specific, but style independent code generators functions.
  */
@@ -704,8 +782,10 @@ std::string CCodeGen::ARRAY_TYPE( unsigned long maxVal )
 		return "unsigned char";
 	else if ( maxVal <= RL_USHORT_MAX )
 		return "unsigned short";
-	else 
+	else if ( maxVal <= RL_UINT_MAX )
 		return "unsigned int";
+	else 
+		return "unsigned long";
 }
 
 std::string CCodeGen::NULL_ITEM()
@@ -739,11 +819,61 @@ std::string CCodeGen::ALPH_TYPE()
 		return "int";
 	case AT_UnsignedInt:
 		return "unsigned int";
+	case AT_Long:
+		return "long";
+	case AT_UnsignedLong:
+		return "unsigned long";
 	}
 	return "";
 }
 
-
+/* Emit the alphabet data type. */
+std::string CCodeGen::WIDE_ALPH_TYPE()
+{
+	if ( redFsm->keyOps->gt( maxKey, redFsm->keyOps->highKey ) ) {
+		if ( cgd->isAlphSigned() ) {
+			if ( redFsm->keyOps->lteq( maxKey, RL_CHAR_MAX ) )
+				return "char";
+			else if ( redFsm->keyOps->lteq( maxKey, RL_SHORT_MAX ) )
+				return "short";
+			else if ( redFsm->keyOps->lteq( maxKey, RL_INT_MAX ) )
+				return "int";
+			else 
+				return "long";
+		}
+		else {
+			if ( redFsm->keyOps->lteq( maxKey, RL_UCHAR_MAX ) )
+				return "unsigned char";
+			else if ( redFsm->keyOps->lteq( maxKey, RL_USHORT_MAX ) )
+				return "unsigned short";
+			else if ( redFsm->keyOps->lteq( maxKey, RL_UINT_MAX ) )
+				return "unsigned int";
+			else 
+				return "unsigned long";
+		}
+	}
+	else {
+		switch ( cgd->alphType ) {
+		case AT_Char:
+			return "char";
+		case AT_UnsignedChar:
+			return "unsigned char";
+		case AT_Short:
+			return "short";
+		case AT_UnsignedShort:
+			return "unsigned short";
+		case AT_Int:
+			return "int";
+		case AT_UnsignedInt:
+			return "unsigned int";
+		case AT_Long:
+			return "long";
+		case AT_UnsignedLong:
+			return "unsigned long";
+		}
+	}
+	return "";
+}
 
 /*
  * D Specific
@@ -794,6 +924,27 @@ std::string DCodeGen::ALPH_TYPE()
 		return "wchar";
 	case AT_Int:
 	case AT_UnsignedInt:
+	case AT_Long:
+	case AT_UnsignedLong:
+		return "dchar";
+	}
+	return "";
+}
+
+/* Emit the alphabet data type for the D language. */
+std::string DCodeGen::WIDE_ALPH_TYPE()
+{
+	switch ( cgd->alphType ) {
+	case AT_Char:
+	case AT_UnsignedChar:
+		return "char";
+	case AT_Short:
+	case AT_UnsignedShort:
+		return "wchar";
+	case AT_Int:
+	case AT_UnsignedInt:
+	case AT_Long:
+	case AT_UnsignedLong:
 		return "dchar";
 	}
 	return "";
