@@ -486,21 +486,21 @@ void TabCodeGen::LOCATE_TRANS()
 		"\n"
 		"	_slen = " << SL() << "[" << CS() << "];\n"
 		"	if ( _slen > 0 ) {\n"
-		"		" << ALPH_TYPE() << POINTER() << "_lower = _keys;\n"
-		"		" << ALPH_TYPE() << POINTER() << "_mid;\n"
-		"		" << ALPH_TYPE() << POINTER() << "_upper = _keys + _slen - 1;\n"
+		"		" << WIDE_ALPH_TYPE() << POINTER() << "_lower = _keys;\n"
+		"		" << WIDE_ALPH_TYPE() << POINTER() << "_mid;\n"
+		"		" << WIDE_ALPH_TYPE() << POINTER() << "_upper = _keys + _slen - 1;\n"
 		"		while (1) {\n"
 		"			if ( _upper < _lower )\n"
 		"				break;\n"
 		"\n"
 		"			_mid = _lower + ((_upper-_lower) >> 1);\n"
-		"			if ( " << GET_KEY() << " < *_mid )\n"
+		"			if ( " << GET_WIDE_KEY() << " < *_mid )\n"
 		"				_upper = _mid - 1;\n"
-		"			else if ( " << GET_KEY() << " > *_mid )\n"
+		"			else if ( " << GET_WIDE_KEY() << " > *_mid )\n"
 		"				_lower = _mid + 1;\n"
 		"			else {\n"
 		"				_trans += (_mid - _keys);\n"
-		"				goto match;\n"
+		"				goto _match;\n"
 		"			}\n"
 		"		}\n"
 		"		_keys += _slen;\n"
@@ -509,21 +509,21 @@ void TabCodeGen::LOCATE_TRANS()
 		"\n"
 		"	_rlen = " << RL() << "[" << CS() << "];\n"
 		"	if ( _rlen > 0 ) {\n"
-		"		" << ALPH_TYPE() << POINTER() << "_lower = _keys;\n"
-		"		" << ALPH_TYPE() << POINTER() << "_mid;\n"
-		"		" << ALPH_TYPE() << POINTER() << "_upper = _keys + (_rlen<<1) - 2;\n"
+		"		" << WIDE_ALPH_TYPE() << POINTER() << "_lower = _keys;\n"
+		"		" << WIDE_ALPH_TYPE() << POINTER() << "_mid;\n"
+		"		" << WIDE_ALPH_TYPE() << POINTER() << "_upper = _keys + (_rlen<<1) - 2;\n"
 		"		while (1) {\n"
 		"			if ( _upper < _lower )\n"
 		"				break;\n"
 		"\n"
 		"			_mid = _lower + (((_upper-_lower) >> 1) & ~1);\n"
-		"			if ( " << GET_KEY() << " < _mid[0] )\n"
+		"			if ( " << GET_WIDE_KEY() << " < _mid[0] )\n"
 		"				_upper = _mid - 2;\n"
-		"			else if ( " << GET_KEY() << " > _mid[1] )\n"
+		"			else if ( " << GET_WIDE_KEY() << " > _mid[1] )\n"
 		"				_lower = _mid + 2;\n"
 		"			else {\n"
 		"				_trans += ((_mid - _keys)>>1);\n"
-		"				goto match;\n"
+		"				goto _match;\n"
 		"			}\n"
 		"		}\n"
 		"		_trans += _rlen;\n"
@@ -533,14 +533,14 @@ void TabCodeGen::LOCATE_TRANS()
 
 void TabCodeGen::GOTO( ostream &ret, int gotoDest, bool inFinish )
 {
-	ret << "{" << CS() << " = " << gotoDest << "; goto again;}";
+	ret << "{" << CS() << " = " << gotoDest << "; goto _again;}";
 }
 
 void TabCodeGen::GOTO_EXPR( ostream &ret, InlineItem *ilItem, bool inFinish )
 {
 	ret << "{" << CS() << " = (";
 	INLINE_LIST( ret, ilItem->children, 0, inFinish );
-	ret << "); goto again;}";
+	ret << "); goto _again;}";
 }
 
 void TabCodeGen::CURS( ostream &ret, bool inFinish )
@@ -568,19 +568,19 @@ void TabCodeGen::NEXT_EXPR( ostream &ret, InlineItem *ilItem, bool inFinish )
 void TabCodeGen::CALL( ostream &ret, int callDest, int targState, bool inFinish )
 {
 	ret << "{" << STACK() << "[" << TOP() << "++] = " << CS() << "; " << CS() << " = " << 
-			callDest << "; goto again;}";
+			callDest << "; goto _again;}";
 }
 
 void TabCodeGen::CALL_EXPR( ostream &ret, InlineItem *ilItem, int targState, bool inFinish )
 {
 	ret << "{" << STACK() << "[" << TOP() << "++] = " << CS() << "; " << CS() << " = (";
 	INLINE_LIST( ret, ilItem->children, targState, inFinish );
-	ret << "); goto again;}";
+	ret << "); goto _again;}";
 }
 
 void TabCodeGen::RET( ostream &ret, bool inFinish )
 {
-	ret << "{" << CS() << " = " << STACK() << "[--" << TOP() << "]; goto again;}";
+	ret << "{" << CS() << " = " << STACK() << "[--" << TOP() << "]; goto _again;}";
 }
 
 void TabCodeGen::BREAK( ostream &ret, int targState )
@@ -588,36 +588,6 @@ void TabCodeGen::BREAK( ostream &ret, int targState )
 	ret << "goto _out;";
 }
 
-std::ostream &TabCodeGen::CONTEXT( int contextId )
-{
-	/* Make a vect to fill with context indexed by state. */
-	int numVals = (redFsm->nextStateId>>3)+1;
-	unsigned char *vals = new unsigned char[numVals];
-	memset( vals, 0, sizeof(unsigned char)*numVals );
-
-	/* For each state where the context is present, set the bit. */
-	for ( RedStateList::Iter st = redFsm->stateList; st.lte(); st++ ) {
-		if ( st->contextSet.find( contextId ) )
-			vals[st->id>>3] |= 1 << (st->id&0x7);
-	}
-
-	/* Write out context info. */
-	for ( int i = 0; i < numVals; ) {
-		out << (unsigned int)vals[i];
-
-		i += 1;
-		if ( i < numVals ) {
-			out << ", ";
-			if ( i % IALL == 0 )
-				out << "\n\t";
-		}
-	}
-	out << "\n";
-
-	delete[] vals;
-	return out;
-}
-
 void TabCodeGen::writeOutData()
 {
 	/* If there are any transtion functions then output the array. If there
@@ -634,7 +604,7 @@ void TabCodeGen::writeOutData()
 	CLOSE_ARRAY() <<
 	"\n";
 
-	OPEN_ARRAY( ALPH_TYPE(), K() );
+	OPEN_ARRAY( WIDE_ALPH_TYPE(), K() );
 	KEYS();
 	CLOSE_ARRAY() <<
 	"\n";
@@ -722,12 +692,6 @@ void TabCodeGen::writeOutData()
 			"static int " << ERROR() << " = " << ERROR_STATE() << ";\n"
 			"\n";
 	}
-
-	for ( ContextMap::Iter ctx = cgd->contextMap; ctx.lte(); ctx++ ) {
-		out << "unsigned char " << CTXDATA() << "_" << ctx->key << "[] = {\n\t";
-		CONTEXT( ctx->value->id );
-		out << "};\n\n";
-	}
 }
 
 void TabCodeGen::writeOutExec()
@@ -743,6 +707,9 @@ void TabCodeGen::writeOutExec()
 		";\n"
 		"	" << UINT() << " _trans;\n";
 
+	if ( anyConditions() )
+		out << "	" << WIDE_ALPH_TYPE() << " _widec;\n";
+
 	if ( anyActions() ) {
 		out << 
 			"	" << ARRAY_TYPE(maxActArrItem) << " *_acts;\n"
@@ -750,16 +717,16 @@ void TabCodeGen::writeOutExec()
 	}
 
 	out <<
-		"	" << ALPH_TYPE() << " *_keys;\n"
+		"	" << WIDE_ALPH_TYPE() << " *_keys;\n"
 		"\n";
 
 	out << 
 		"	" << P() << " -= 1;\n"
-		"	goto resume;\n"
+		"	goto _resume;\n"
 		"\n";
 
 	/* Head of the loop. */
-	out << "again:\n";
+	out << "_again:\n";
 
 	if ( anyToStateActions() ) {
 		out <<
@@ -774,7 +741,7 @@ void TabCodeGen::writeOutExec()
 			"\n";
 	}
 
-	out << "resume:\n";
+	out << "_resume:\n";
 
 	if ( cgd->hasEnd ) {
 		out << 
@@ -799,6 +766,9 @@ void TabCodeGen::writeOutExec()
 			"\n";
 	}
 
+	if ( anyConditions() )
+		COND_TRANSLATE();
+
 	LOCATE_TRANS();
 
 	if ( redFsm->errState != 0 ) {
@@ -807,7 +777,7 @@ void TabCodeGen::writeOutExec()
 			"		goto _out;\n";
 	}
 
-	out << "match:\n";
+	out << "_match:\n";
 
 	if ( anyRegCurStateRef() )
 		out << "	_ps = " << CS() << ";\n";
@@ -822,7 +792,7 @@ void TabCodeGen::writeOutExec()
 	if ( anyRegActions() ) {
 		out <<
 			"	if ( " << TA() << "[_trans] == 0 )\n"
-			"		goto again;\n"
+			"		goto _again;\n"
 			"\n"
 			"	_acts = " << ARR_OFF( A(), TA() + "[_trans]" ) << ";\n"
 			"	_nacts = " << CAST(UINT()) << " *_acts++;\n"
@@ -835,7 +805,7 @@ void TabCodeGen::writeOutExec()
 			"\n";
 	}
 
-	out << "	goto again;\n";
+	out << "	goto _again;\n";
 	
 	if ( cgd->hasEnd || anyRegBreak() || redFsm->errState != 0 )
 		out << "	_out: {}\n";
